#####################################################################
# NAME:   graph2.tcl
# AUTHOR: Clif Flynt
# DATE:   10/2005
# DESC:   Graphing library
#         
# PARAMETERS:  
#  many and varied
#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2005
#
#   Contact: clif@noucorp.com
#
#####################################################################

# MUNG: 
#   Add support for $dset,tag for individual point tags
#   LONGERTERM: Change 0,foo to FOO for internally generated values.
#   Maybe binding for points, $dset,bind {list...}
set release(graph2.tcl) {$Header: /home/cvs/dataGraph/graph2.tcl,v 1.2 2005/10/02 13:23:38 clif Exp $}
namespace eval graph2 {
  variable State
  # Support for multiple independent data sets on a graph.
  #  0 for overall
  #  1-N for independant data sets (independant axis).  Overrides 0.

  # State_$cvs(style) barchart, line, scatter, pie, area, colorbar
  # State_$cvs(script) A script to evaluate after drawing bar components.

  # State_$cvs(colors) A list of colors for pie charts or colorbars
  # State_$cvs(color,min) Min value to scale for colors in colorbar
  #        lindex $State(colors) [expr\
    {int(($val-$min)/[llength $State(colors)])}]
  # State($colorPosition,colorUbsed) 1/0 if a color has been used in this bar

  # State_$cvs(rotate)  0 for Normal, 1 for horizontal histogram bars.
  # State_$cvs(histStyle)   single/stacked for a single line or stacked bars
  #                        at a location      

  # State_$cvs(histShowAbsolute) If != 0, display the absolute value
  #                              at the end of the bar.  Only valid
  #                              for single dataset plots.

  # Button 1 has default bindings to show info (scatter/line, bar, pie) 
  # Button 3 has default bindings to move (pie)
  # State_$cvs(bind,element,event)     Script to bind to events
  #           element - line, point, bar, slice, xaxis, yaxis
  #           event   - Any X-11/Tk event
  #           script: A script to bind to the event.  
  #                     Will Append:  cvs Xpos Ypos x_orig y_orig id
  #                     Default for <ButtonPress-1> is Popup variant.
  #          
  #
  # State_$cvs(plotarea,l) Left edge of plot area (after any axis plotted)
  # State_$cvs(plotarea,r) Right edge of plot area (after any axis plotted)
  # State_$cvs(plotarea,t) Top edge of plot area (after any axis plotted)
  # State_$cvs(plotarea,b) Bottom edge of plot area (after any axis plotted)

  # State_$cvs(#,x) raw X values
  # State_$cvs(#,y) raw Y values
  # State_$cvs(#,z) raw Z values - For use with colorbar
  # State_$cvs(#,tagList) A list of tags to apply to points, for use with
		# scatter and line graphs
  # State_$cvs(#,color) color to draw elements/lines
  # State_$cvs(#,dash) a dash pattern for this line - any Tk format OK.
  # State_$cvs(#,width) width for a line
  # State_$cvs(#,drawPoint)     1 to draw points, o to not draw points.
  #                             Only valid for line graphs, default=1
  # State_$cvs(#,point) What to draw at points - x + o box filled-o filled-box\
    {}
  #            may be a proc name, which will be invoked as $proc $cvs $x $y
  # State_$cvs(#,selected)  1/0 for whether this dataset is selected.
  #                        Unselected datasets are not graphed, and
  #                        are grayed in the legend
  #                        Default is 1


  # State_$cvs(axis,y,label)    Label for the Y axis
  # State_$cvs(axis,y,nameProc) A proc to return a label for a given location\
    - proc f {scrarg num} {...}
  # State_$cvs(axis,y,font)  - Font to use for y axis text
  # State_$cvs(axis,y,format)- Format string to use to generate default labels
  # State_$cvs(0,axis,y,use) - Turn on/off entire axis
  # State_$cvs(0,axis,y,Min) - A forced minimal value for the axis
  # State_$cvs(0,axis,y,Max) - A forced minimal value for the axis
  # State_$cvs(0,axis,y,TopMargin)  A top margin to leave (dflt 10 pixels)
  # State_$cvs(0,axis,y,BottomMargin)  A bottom margin to leave (dflt 20 pixels)
  # State_$cvs(#,axis,y,use) - Whether to include this dataset in axis
  #                           calculations.  If 0, the dataset will be
  #                           scaled to fit the canvas, not to match other
  #                           data.
  #                           if == 0, State_$cvs(#,axis,y,minTick), etc
  #                           are valid for this entry.  If == 1, use the
  #                           mintick,etc in State_$cvs(0,axis,y,minTick)
  #                Ditto for X axis.

  # State_$cvs(#,axis,y,use)      0/1 to show axis
  # State_$cvs(#,axis,y,width)    Draw with width (0=no lines)
  # State_$cvs(#,axis,y,dash)     Draw with dash
  # State_$cvs(#,axis,y,color)    Color for axis 
  # State_$cvs(#,axis,y,lColor)   Color for axis labels: dflt black, {} no label
  # State_$cvs(#,axis,y,minTick)  Smallest tick mark
  # State_$cvs(#,axis,y,maxTick)  Largest tick mark
  # State_$cvs(#,axis,y,stepTick) Steps per tick mark
  # State_$cvs(axis,y,stepTick)   Program declared tick step size override

  # State_$cvs(axis,x,label)      Label for the Y axis
  # State_$cvs(axis,x,nameProc)   A proc to return a label for a given
  #                               location - proc f {scrarg num} {...}
  # State_$cvs(axis,x,font)       Font to use for x axis text
  # State_$cvs(axis,x,format)- Format string to use to generate default labels
  # State_$cvs(#,axis,x,use)      0/1 to show axis
  # State_$cvs(#,axis,x,width)    Draw with width (0=no lines)
  # State_$cvs(#,axis,x,dash)     Draw with dash
  # State_$cvs(#,axis,x,color)    Color for axis 
  # State_$cvs(#,axis,x,lColor)   Color for axis labels: dflt black, {} no label
  # State_$cvs(0,axis,x,Min)      A forced minimal value for the axis
  # State_$cvs(0,axis,x,Max)      A forced maximum value for the axis
  # State_$cvs(0,axis,x,LeftMargin)   A left margin to leave (dflt 30 pixels)
  # State_$cvs(0,axis,x,RightMargin)   A right margin to leave (dflt 10 pixels)
  # State_$cvs(#,axis,x,minTick)  Smallest tick mark
  # State_$cvs(#,axis,x,maxTick)  Largest tick mark
  # State_$cvs(#,axis,x,stepTick) Steps per tick mark
  # State_$cvs(axis,x,stepTick)   Program declared tick step size override

  # State_$cvs(legend,use)      0/1 to show legend
  # State_$cvs(legend,font)     Font to use for legend {default arial 10}
  # State_$cvs(#,legend,title)  Title for # element on graph
  # X/Y/Anchor as for relx, rely, anchor with place command.
  # State_$cvs(legend,relx)        X location for legend as percent.
  # State_$cvs(legend,rely)        Y location for legend as percent.
  # State_$cvs(legend,x)           X location for legend as pixels.
  # State_$cvs(legend,y)           Y location for legend as pixels.
  # State_$cvs(legend,X)           X runtime location (after user moves it)
  # State_$cvs(legend,Y)           Y runtime location (after user moves it)
  # State_$cvs(legend,anchor)   	  Location for anchor (news)

  proc + {a b} {return [expr {$a + $b}]}
  proc - {a b} {return [expr {$a - $b}]}
  proc * {a b} {return [expr {$a * $b}]}

  set unique 0

  proc unique {} {
    variable unique
    return [incr unique]
  }

  # Reduce a set of data to a reasonable number of points
  #  to display on a screen.
  proc reduceData {data} {
    puts "reduceData NOT Implemented"
    exit
  }

  proc configure {cvs name {value {NONE}}} {
    upvar [namespace current]::State_$cvs State

    if {[string equal $value NONE]} {
      return $State($name)
    }
    if {[string first - $name] == 0} {
      set name [string range $name 1 end]
    }
    if {[string first "bind," $name] == 0} {
      set State($name) $value
    }\
    elseif {[info exists State($name)]} {
      set State($name) $value
    } else {
      error "Bad field: $name - should be one of [lsort [array names State]]"
    }
  }

  proc getTickSize {min max {numSteps 10}} {
    if {$numSteps == 0} {set numSteps 1}

    set stepsize [expr {($max - $min) / double($numSteps)}]
    # puts "STEP: $stepsize -- $max -- $min -- $numSteps "
    if {$stepsize == 0} {set stepsize 1}

    set mult [expr {pow(10,(int(log10(abs($stepsize)))))}]

    foreach t {.1 .2 .5 1 2 5 10 20 50} {
      if {[set tick [expr {$t * $mult}]] >= $stepsize} {break}
    }

    set tmin [expr {(int($min/$tick)) * $tick}]
    set tmax [expr {(int($max/$tick)) * $tick}]

    if {($max > 0) &&($max > $tmax)} {
      set tmax [expr {$tmax + $tick}]
    }

    if {[expr {$tmin - $tick}] == 0} {set tmin 0}

    if {($min < 0) &&($min < $tmin)} {
      set tmin [expr {$tmin - $tick}]
    }

    if {($min == $tmin) &&($min != 0)} {
      set tmin [expr {$min - $tick}]
    }

    if {($max == $tmax) &&($max != 0)} {
      #        set tmax [expr {$max + ($tick/2)}]
      set tmax [expr {$max + $tick}]
    }

    return [list $tick $tmin $tmax]

  }

  proc SetLocation {x y} {
    variable mouseX
    variable mouseY
    set mouseX $x
    set mouseY $y
  }

  proc ClearLocation {} {
    variable mouseX
    variable mouseY
    catch {unset mouseX
      unset mouseY}
  }


  proc MoveItem {cvs id x y} {
    upvar [namespace current]::State_$cvs State
    variable mouseX
    variable mouseY

    if {![info exists mouseX]} {
      set mouseX $x
      set mouseY $y
      return
    }

    set xoff [expr {$x - $mouseX}]
    set yoff [expr {$y - $mouseY}]
    $cvs move $id $xoff $yoff
    foreach {xo yo} [$cvs coords $id] {break}
    set mouseX $x
    set mouseY $y
    set State(legend,X) $xo
    set State(legend,Y) $yo
  }

  ################################################################
  # proc PieChartMovePie {win id x y}--
  #    Moves an item on a canvas - specificly a pie chart.
  # 
  # Arguments
  #   win		Name of window containing items to move
  #   id		Tag or Id of items to move
  #   x/y		From binding
  # 
  # Results
  #   Global variables xPieMouse and yPieMouse are modified
  #   The canvas is modified

  proc PieChartMovePie {cvs x y v1 v2 id} {
    # {cvs id x y} 
    global xPieMouse yPieMouse
    set id [lindex [$cvs itemcget $id -tags] 0]
    $cvs move $id [expr $x - $xPieMouse] [expr $y - $yPieMouse]
    set xPieMouse $x
    set yPieMouse $y
  }

  proc PieChartMoveSlice {cvs x y v1 v2 id} {
    # {cvs id x y} 
    global xPieMouse yPieMouse
    $cvs move $id [expr $x - $xPieMouse] [expr $y - $yPieMouse]
    set xPieMouse $x
    set yPieMouse $y
  }

  proc PieChartSetPie {cvs x y v1 v2 id} {
    global xPieMouse yPieMouse
    set xPieMouse $x
    set yPieMouse $y
  }

  ################################################################
  # proc PieChartDrawPie {win tag x1 y1 radius slices}--
  #    Draw a piechart
  # Arguments
  # win 	The Canvas to be drawn in
  # tag   A tag to apply to the graphics drawn, so they can be moved en-mass
  # x1 	X offset for left side of bbox
  # y1 	Y offset for top of bbox
  # radius  Radius of the Pie (NOTE - *NOT* diameter)
  # slices  List of slices {label-text percent-of-pie color-of-slice}
  # 
  # Results
  #   Canvas is modified, bindings are added
  # 
  proc PieChartDrawPie {cvs tag x1 y1 radius slices} {
    upvar [namespace current]::State_$cvs State
    set next 0
    set x0 [expr $radius * 1.1 + $x1]
    set y0 [expr $radius * 1.1 + $y1]

    set dim [list [expr $x0 - $radius] [expr $y0 - $radius] [expr $x0 +\
      $radius] [expr $y0 + $radius]]

    set tag $tag[unique]

    foreach slice $slices {
      foreach {label percentage color} $slice {
	set degrees [expr $percentage * 360]
	set id [eval $cvs create arc $dim -start $next -extent $degrees\
	  -fill $color -width 2 -outline black -tag [list [list $tag\
	  $tag.$color]]]

	foreach indx [array names State bind,slice,*] {
	  foreach {bnd ln event} [split $indx ,] {break}
	  if {![string equal {} $State($indx)]} {
	    $cvs bind $id $event "$State($indx) $cvs %x %y\
	      {$label} $percentage  $id"
	  }
	}
      }
      set next [expr $next + $degrees]
    }

    #    $cvs bind $tag <ButtonPress-1> {set xPieMouse %x; set yPieMouse %y}
    #    $cvs bind $tag <B1-Motion> "[namespace current]::PieChartMovePie $cvs\
      $tag %x %y"
    return $tag
  }

  proc DrawXAxis {cvs} {
    upvar [namespace current]::State_$cvs State

    if {[winfo ismapped $cvs]} {
      set border [expr {[$cvs cget -borderwidth] * 2 + 2}]
      set w [expr {[winfo width $cvs] - $border}]
      set h [expr {[winfo height $cvs] - $border}]
    } else {
      set w [$cvs cget -width]
      set h [$cvs cget -height]
    }

    # Override w/h with scrollregion if it's defined
    # Assume 0 based canvases, nothing silly in negative 

    foreach {l t r b} [$cvs cget -scrollregion] {break}
    if {[info exists b]} {
      if {$b > $h} {set h $b}
      if {$r > $w} {set w $r}
    }

    set bbox [$cvs bbox yAxis]
    if {$bbox ne ""} {
      foreach {l t r b} $bbox {break}
    } else {
      set b [- $h $State(0,axis,y,BottomMargin)]
      set t $State(0,axis,y,TopMargin)
    }

    set r [- $w $State(0,axis,x,RightMargin)]
    set l $State(0,axis,x,LeftMargin)

    set len [- $r $l]

    set tickCount [expr {($State(0,axis,x,maxTick) - $State(0,axis,x,minTick))\
      / $State(0,axis,x,stepTick)}]

    if {$tickCount == 0} {set tickCount 1}

    set tickPixels [expr {$len / $tickCount}]

    set pix $l

    set font $State(axis,x,font)
    set bottom $b
    set gColor $State(0,axis,x,color)
    set lColor $State(0,axis,x,lColor)
    set State(plotarea,l) $l
    set State(plotarea,r) $r

    set h [+ $bottom 2]

    for {set p $State(0,axis,x,minTick)} {$p <= $State(0,axis,x,maxTick)}\
      {set p [expr {$p + $State(0,axis,x,stepTick)}]} {

      if {[string equal "" $State(axis,x,nameProc)]} {
	set lbl [format $State(axis,x,format) $p]
      } else {
	set lbl [eval $State(axis,x,nameProc) $p]
      }

      $cvs create text [- $pix 2] $h -text $lbl -anchor n -tags xAxis\
	-fill $lColor -font $font
      set item [$cvs create line $pix $bottom $pix $t -fill $gColor\
	-dash $State(0,axis,x,dash)]
      set pix [expr {$pix + $tickPixels}]

      foreach indx [array names State bind,xaxis,*] {
	foreach {bnd ln event} [split $indx ,] {break}
	if {![string equal {} $State($indx)]} {
	  $cvs bind $item $event "$State($indx) $cvs %x %y $lbl $lbl $item"
	}
      }
    }

    if {![string equal "" $State(axis,x,label)]} {
      foreach {fam pt style} $font {break}
      incr pt 2
      set font [list $fam $pt bold]
      set ls [font metric $font -linespace]
      incr ls 2
      $cvs create text [expr {($l + $r) /2}] [+ $h $ls]\
	-text $State(axis,x,label) -anchor n -font $font
    }
  }

  proc DrawYAxis {cvs} {
    upvar [namespace current]::State_$cvs State

    if {[winfo ismapped $cvs]} {
      set border [expr {[$cvs cget -borderwidth] * 2 + 2}]
      set w [expr {[winfo width $cvs] - $border}]
      set h [expr {[winfo height $cvs] - $border}]
    } else {
      set w [$cvs cget -width]
      set h [$cvs cget -height]
    }

    # Override w/h with scrollregion if it's defined
    # Assume 0 based canvases, nothing silly in negative 

    foreach {l t r b} [$cvs cget -scrollregion] {break}
    if {[info exists b]} {
      if {$b > $h} {set h $b}
      if {$r > $w} {set w $r}
    }

    set bbox [$cvs bbox xAxis]
    if {$bbox ne ""} {
      foreach {l t r b} $bbox {break}
    } else {
      set l $State(0,axis,x,LeftMargin)
      set r [- $w $State(0,axis,x,RightMargin)]
    }
    set b [- $h $State(0,axis,y,BottomMargin)]
    set t $State(0,axis,y,TopMargin)

    set len [- $b $t]

    set tickCount [expr {ceil(($State(0,axis,y,maxTick)\
      - $State(0,axis,y,minTick)) / $State(0,axis,y,stepTick))}]

    set tickCount [expr {($State(0,axis,y,maxTick) - $State(0,axis,y,minTick))\
      / $State(0,axis,y,stepTick)}]

    if {$tickCount == 0} {set tickCount 1}
    set tickPixels [expr {$len / $tickCount}]

    set font $State(axis,y,font)

    set rt [font measure $font $State(0,axis,y,minTick)]

    set pix [- $h $State(0,axis,y,BottomMargin)]

    set gColor $State(0,axis,y,color)
    set lColor $State(0,axis,y,lColor)

    set State(plotarea,t) $t
    set State(plotarea,b) $b

    set first 1

    # Add a smidgen to the max value to account for floating pt
    #  errors that can cause the top line to be skipped.
    #  for {set p .8} {$p < 4.6} {incr p 0.2} - stops at 4.4

    for {set p $State(0,axis,y,minTick)} {$p <= [expr {.00001 +\
      $State(0,axis,y,maxTick)}]} {set p [expr {$p +\
      $State(0,axis,y,stepTick)}]} {

      #	  if {($p == $State(0,axis,y,maxTick)) && !$first} {continue}
      set first 0

      if {[string equal "" $State(axis,y,nameProc)]} {
	set lbl [format $State(axis,y,format) $p]
      } else {
	set lbl [eval $State(axis,y,nameProc) $p]
      }

      $cvs create text [- $State(0,axis,x,LeftMargin) 3] $pix -text $lbl\
	-anchor se -tags yAxis -fill $lColor -font $font

      set item [$cvs create line [- $State(0,axis,x,LeftMargin) 10] $pix $r\
	$pix -fill $gColor -dash $State(0,axis,y,dash)]

      set pix [expr {$pix - $tickPixels}]

      foreach indx [array names State bind,yaxis,*] {
	foreach {bnd ln event} [split $indx ,] {break}
	if {![string equal {} $State($indx)]} {
	  $cvs bind $item $event "$State($indx) $cvs %x %y $lbl $lbl $item"
	}
      }
    }

    if {![string equal "" $State(axis,y,label)]} {
      foreach {fam pt style} $font {break}
      incr pt 2
      set font [list $fam $pt bold]
      set ls [font metric $font -linespace]
      incr ls 2
      set lines [llength [split $State(axis,y,label)]]
      set ypos [expr {(($t + $b + $b) / 3) -($lines * $ls / 3.0)}]
      foreach word $State(axis,y,label) {
	$cvs create text 5 $ypos -text $word -anchor w -font $font
	set ypos [expr {$ypos + $ls}]
      }
    }
  }

  proc initStateDefaults {arrName} {
    upvar $arrName State
    array set State {
        style scatter 
	colors {red orange yellow green blue violet} 
	bind,bar,<ButtonPress-1>       graph2::Popup      
	bind,point,<ButtonPress-1>     graph2::Popup 
	bind,line,<ButtonPress-1>      graph2::PopupByPlotArea 
	bind,slice,<ButtonPress-1>     graph2::Popup
        bind,slice,<ButtonPress-3>     graph2::PieChartSetPie 
	bind,slice,<B3-Motion>         graph2::PieChartMovePie 
	bind,slice,<Control-B3-Motion> graph2::PieChartMoveSlice 
	bind,xaxis,<ButtonPress-3>    "graph2::StartMinMax x" 
	bind,xaxis,<B3-Motion>        "graph2::MoveMinMax x"
        bind,xaxis,<ButtonRelease-3>  "graph2::EndMinMax x"
        bind,yaxis,<ButtonPress-3>    "graph2::StartMinMax y"
        bind,yaxis,<B3-Motion>        "graph2::MoveMinMax y"
        bind,yaxis,<ButtonRelease-3>  "graph2::EndMinMax y" 
	color,min {} 
	script    {} 
	0,legend,title {} 
	0,color black 
	0,dash {{}} 
	0,width {1} 
	0,point +
        0,drawPoint 1 
	0,selected 1 
	0,axis,y,use 1 
	0,axis,y,width 1 
	0,axis,y,dash  {} 
	0,axis,y,color black 
	0,axis,y,lColor black 
	0,axis,y,Min  {} 
	0,axis,y,Max  {} 
	0,axis,y,BottomMargin 20 
	0,axis,y,TopMargin 25
        0,axis,y,minTick {} 
	0,axis,y,maxTick {} 
	0,axis,y,stepTick  {} 
	axis,y,stepTick {} 
	axis,y,label {} 
	axis,y,nameProc {} 
	axis,y,font     {arial 10} 
	axis,y,format {%.2f} 
	0,axis,x,use 1 
	0,axis,x,width 1
        0,axis,x,dash {} 
	0,axis,x,color black 
	0,axis,x,lColor black 
	0,axis,x,Min {} 
	0,axis,x,Max {} 
	0,axis,x,LeftMargin 50 
	0,axis,x,RightMargin 10
        0,axis,x,minTick  {} 
	0,axis,x,maxTick  {} 
	0,axis,x,stepTick {} 
	0,tagList         {}
	axis,x,stepTick {} 
	axis,x,label {} 
	wid 2 
	rotate 0 
	histStyle stacked
        histShowAbsolute 0 
	axis,x,nameProc {} 
	axis,x,font {arial 10}
        axis,x,format {%.2f} 
	legend,font {arial 10} 
	legend,use 1 
	legend,anchor nw 
	legend,x 5 
	legend,y 5 
	legend,relx {} 
	legend,rely {} 
	legend,X    {} 
	legend,Y    {}
	}
  }

  proc create {name args} {
    upvar [namespace current]::State_$name State

    bind $name <Destroy> "[namespace current]::clearGraph $name"
    
    initStateDefaults State

    if {[set p [lsearch $args "-style"]] >= 0} {
      set rot 0
      if {[set p2 [lsearch $args "-rotate"]] >= 0} {
	set rot [lindex $args [incr p2]]
      }
      incr p
      switch [lindex $args $p] {
      area {
	  set State(0,point) "filled-bigbox"
	}
      histogram {
	  set State(0,point) "filled-bigbox"
	  if {$rot} {
	    set State(0,axis,y,color) [$name cget -background]
	  } else {
	    set State(0,axis,x,color) [$name cget -background]
	  }
	}
      pie {
	  set State(legend,font) {arial 12}
	}
      }
    }
    foreach {k v} $args {
      set k [string range $k 1 end]
      if {[string equal {} [set ind [array names State "$k*"]]]} {
	set ind [array names State "*,$k*"]
      }
      if {[string first "bind," $k] == 0} {
	set ind $k
      }
      if {[string equal {} $ind]} {error "bad value: $k -- ..$ind.."}
      set State($ind) $v
    }

    if {[string equal $State(style) "area"]} {
      set State(histStyle) "single"
    }

    bind $name <Configure> "[namespace current]::drawGraph $name"

    if {![string equal {} $State(axis,x,label)]} {
      incr State(0,axis,y,BottomMargin) 20
    }
  }

  proc SetTicks {cvs axis data len {set 0}} {
# puts "SetTicks $cvs $axis $data $len $set"
    upvar [namespace current]::State_$cvs State
    foreach {min max} [MinMax $data] {break}

    #    if {$max == $min} {
    #      set cnt [string length [format "$State(axis,$axis,format)" $max]]
    #    } else {
    #      set cnt1 [expr {abs(int(log10($max-$min)))}]
    #      set cnt2 [expr {abs(int(log10($max)))}]
    #      if {$cnt2 > $cnt1} {
    #        set cnt $cnt2
    #      } else {
    #        set cnt $cnt1
    #      }
    #    }

    #    set fail [catch {expr {abs(int(log10($max-$min)))}} cnt]
    #    if {$fail} {
    #      puts "FAILED ON: abs(int(log10($max-$min)))"
    #      set cnt 5
    #    }

    set cnt [string length [format "$State(axis,$axis,format)" $max]]

    if {[string match $axis "x"]} {
      # set numTicks [expr {$len / ([font measure $State(axis,x,font) n] *\
	($cnt+1))}]
      set numTicks [expr {$len /([font measure $State(axis,x,font) m] * $cnt)}]
    } else {
      # set numTicks [expr {$len / (3*[font measure $State(axis,y,font) m])}]
      set numTicks [expr {$len /(1.5*[font metrics $State(axis,y,font)\
	-linespace])}]
    }

#    if {[string equal "histogram" $State(style)]} {
#      set min 0
#    }

    if {$State($set,axis,$axis,Min) ne ""} {
      set min $State($set,axis,$axis,Min)
    }

    if {$State($set,axis,$axis,Max) ne ""} {
      set max $State($set,axis,$axis,Max)
    }

    foreach {steptick minTick maxTick} \
        [getTickSize $min $max $numTicks] {break}

    if {![string match {} $State(axis,$axis,stepTick)]} {
      set steptick $State(axis,$axis,stepTick)
    }
    foreach var1 [list \
              State($set,axis,$axis,stepTick) \
              State($set,axis,$axis,minTick) State($set,axis,$axis,maxTick)] \
            var2  {steptick minTick maxTick} {
      set $var1 [set $var2]
    }

    return "$steptick $minTick $maxTick"
  }

  proc insertData {cvs xvals yvals args} {
    upvar [namespace current]::State_$cvs State

    if {[set p [lsearch $args -dataset]] >= 0} {
      incr p
      set dataset [lindex $args $p]
      set args [lreplace $args [- $p 1] $p]
    } else {
      set dataset 1
    }

    set State($dataset,legend,title) $dataset

    if {$State(rotate)} {
      eval lappend State($dataset,x) $yvals
      eval lappend State($dataset,y) $xvals
    } else {
      eval lappend State($dataset,x) $xvals
      eval lappend State($dataset,y) $yvals
    }

    set State($dataset,axis,x,Min) {}
    set State($dataset,axis,x,Max) {}
    set State($dataset,axis,y,Min) {}
    set State($dataset,axis,y,Max) {}
    set State($dataset,z) {}

    foreach i {axis,x,use axis,y,use color point drawPoint selected dash\
               width tagList} {
      set State($dataset,$i) $State(0,$i)
    }
# puts "ARGS $args"
    foreach {k v} $args {
      set k [string range $k 1 end]
      set ind [array names State "$dataset*$k"]
# puts "K: ..$k.. IND: $ind"
      set State($ind) $v
    }
  }

  proc removeDataset {cvs dataset} {
    upvar [namespace current]::State_$cvs State
    foreach index [array names State $dataset,*] {
      unset State($index)
    }
  }

  proc removeAllDatasets {cvs} {
    upvar [namespace current]::State_$cvs State

    foreach index [array names State *,x] {
      if {[string first "legend" $index] >=0} {continue}
      foreach {dset x} [split $index ,] {break}
      if {$dset > 0} {
	removeDataset $cvs $dset
      }
    }
  }

  proc MinMax {data} {
    # puts "MinMax: $data [info level -1] --- [info level -2]"
    set max [lindex $data 0]
    set min [lindex $data 0]
    foreach d $data {
      if {$d > $max} {set max $d}
      if {$d < $min} {set min $d}
    }
    # puts "MINMAX: [list $min $max] - $data"
    return [list $min $max]
  }
  
  proc MakeScaleProcs {cvs num} {
    upvar [namespace current]::State_$cvs State

    set xl $State(plotarea,l)
    set xr $State(plotarea,r)
    set yt $State(plotarea,t)
    set yb $State(plotarea,b)

    set xpixels [expr {$xr - $xl}]
    set ypixels [expr {$yb - $yt}]
    
    if {$State($num,axis,y,use)} {
      set minTickY $State(0,axis,y,minTick)
      set maxTickY $State(0,axis,y,maxTick)
    } else {
      set minTickY $State($num,axis,y,minTick)
      set maxTickY $State($num,axis,y,maxTick)
    }

    if {$State($num,axis,x,use)} {
      set minTickX $State(0,axis,x,minTick)
      set maxTickX $State(0,axis,x,maxTick)
    } else {
      set minTickX $State($num,axis,x,minTick)
      set maxTickX $State($num,axis,x,maxTick)
    }

    set dNX [expr {double($maxTickX - $minTickX)}]
    set dNY [expr {double($maxTickY - $minTickY)}]
    if {$dNX == 0} {set dNX 1}
    if {$dNY == 0} {set dNY 1}
    set dPX [expr {double($xr - $xl)}]
    set dPY [expr {double($yb - $yt)}]
    set MnPx [expr {$dPX * $minTickX}]
    set dNL [expr {$dNX * $xl}]

    set MnPy [expr {$dPY * $minTickY}]
    set dNT [expr {$dNY * $yt}]

    proc scaleX {x} [subst {return \
        \[expr {int((($dPX * \$x) - $MnPx + $dNL) / $dNX)}]
	}]
    proc scaleY {y} [subst {return \
        \[expr {int($yt + $yb -((($dPY * \$y) - $MnPy + $dNT) / $dNY))}]
	}]
  }


  proc DrawPoint {cvs x y style color {tags {}}} {
    switch $style {
    {} {
	# Do nothing for empty dot style
	set id {}
      }
    o {
	set id [$cvs create oval [- $x 2] [- $y 2] [+ $x 2] [+ $y 2]\
	  -outline $color -tags $tags]
      }
    filled-o {
	set id [$cvs create oval [- $x 2] [- $y 2] [+ $x 2] [+ $y 2]\
	  -outline $color -fill $color -tags $tags]
      }
    box {
	set id [$cvs create rectangle [- $x 2] [- $y 2] [+ $x 2] [+ $y 2]\
	  -outline $color -tags $tags]
      }
    filled-box {
	set id [$cvs create rectangle [- $x 2] [- $y 2] [+ $x 2] [+ $y 2]\
	  -outline $color -fill $color -tags $tags]
      }
    filled-bigbox {
	set id [$cvs create rectangle [- $x 3] [- $y 3] [+ $x 3] [+ $y 3]\
	  -outline black -fill $color -tags $tags]
      }
    + {
	set id [$cvs create line [- $x 2] $y [+ $x 3] $y -fill $color\
	  -tags $tags]
	set id [$cvs create line $x [- $y 2] $x [+ $y 3] -fill $color\
	  -tags $tags]
      }
    x {
	set id [$cvs create line [- $x 2] [- $y 2] [+ $x 3] [+ $y 3]\
	  -fill $color -tags $tags]
	set id [$cvs create line [- $x 2] [+ $y 2] [+ $x 3] [- $y 3]\
	  -fill $color -tags $tags]
      }
    dot {
	set id [$cvs create line $x $y $x [+ $y 1] -fill $color -tags $tags]
      }
    default {
	set id [$style $cvs $x $y]
      }
    }
    return $id
  }

  proc drawLines {cvs} {
    upvar [namespace current]::State_$cvs State

    foreach nm [lsort -dict [array names State *,x]] {

      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}

      MakeScaleProcs $cvs $num
      
      foreach xo1 $State($num,x) xo2 [lrange $State($num,x) 1 end] \
          yo1 $State($num,y) yo2 [lrange $State($num,y) 1 end] {
         
	if {[string equal {} $xo2]} {
	  set xo2 $xo1
	  set yo2 $yo1
	}

	set x1 [scaleX $xo1]
	set x2 [scaleX $xo2]
        set y1 [scaleY $yo1]
        set y2 [scaleY $yo2]

	# Sudden death tests
	if {($x2 < $State(plotarea,l)) &&($x1 < $State(plotarea,l))} {continue}
	if {($x2 > $State(plotarea,r)) &&($x1 > $State(plotarea,r))} {continue}
	if {($y2 < $State(plotarea,t)) &&($y1 < $State(plotarea,t))} {continue}
	if {($y2 > $State(plotarea,b)) &&($y1 > $State(plotarea,b))} {continue}

	if {$x1 == $x2} {set x1 [expr {$x1 +($x1 * .000001)}]}

	set slope [expr {($y2 - $y1) / double($x2 - $x1)}]

	# Confirm first point is inside clipping boundary, and add it
	#   else calculate new point and add that instead

	#  formulas y = y2 - slope * (x2-x)
	#           x = x2 - (y2-y) / slope

	if {($x1 <= $State(plotarea,r)) &&($x1 >= $State(plotarea,l)) &&
	    ($y1 >= $State(plotarea,t)) &&($y1 <= $State(plotarea,b))} {
	  set x $x1
	  set y $y1
	} elseif {($x1 < $State(plotarea,l))} {
	  # Off left boundary
	  if {($y1 < $State(plotarea,t))} {
	    # And above top
	    # Check boundary with left edge
	    set x $State(plotarea,l)
	    set y [expr {$y2 - $slope*($x2 - $x)}]
	    if {($y < $State(plotarea,t))} {
	      #  Ooops passes through top boundary, not left
	      set y $State(plotarea,t)
	      set x [expr {$x2 -($y2-$y) /$slope}]
	    }
	  } elseif {($y1 > $State(plotarea,b))} {
	    # And below bottom
	    set x $State(plotarea,l)
	    set y [expr {$y2 - $slope*($x2 - $x)}]
	    if {($y > $State(plotarea,b))} {
	      #  Ooops passes through bottom boundary, not left
	      set y $State(plotarea,b)
	      set x [expr {$x2 -($y2-$y) /$slope}]
	    }
	  } else {
	    # And within Y coords.
	    # Check boundary with left edge
	    set x $State(plotarea,l)
	    set y [expr {$y2 - $slope*($x2 - $x)}]
	  }
	} elseif {($x1 > $State(plotarea,r))} {
	  puts "NO RIGHT SUPPORT YET"
	} elseif {($y1 < $State(plotarea,t))} {
	  # Off top edge
	  if {($x1 < $State(plotarea,l))} {
	    # And off left
	    # Check boundary with top edge
	    set y $State(plotarea,t)
	    set x [expr {$x2 -($y2-$y) /$slope}]
	    if {($x < $State(plotarea,l))} {
	      #  Ooops passes through top boundary, not left
	      set x $State(plotarea,l)
	      set y [expr {$y2 - $slope*($x2 - $x)}]
	    }
	  } elseif {($x1 > $State(plotarea,r))} {
	    # And off to right
	    set y $State(plotarea,t)
	    set x [expr {$x2 -($y2-$y) /$slope}]
	    if {($y > $State(plotarea,b))} {
	      #  Ooops passes through right boundary, not top
	      set x $State(plotarea,r)
	      set y [expr {$y2 - $slope*($x2 - $x)}]
	    }
	  } else {
	    # And within Y coords.
	    # Check boundary with left edge
	    set y $State(plotarea,t)
	    set x [expr {$x2 -($y2-$y) /$slope}]
	  }
	} elseif {($y1 > $State(plotarea,b))} {
	  # Off bottom edge
	  if {($x1 < $State(plotarea,l))} {
	    # And off left
	    # Check boundary with top edge
	    set y $State(plotarea,b)
	    set x [expr {$x2 -($y2-$y) /$slope}]
	    if {($x < $State(plotarea,l))} {
	      #  Ooops passes through top boundary, not left
	      set x $State(plotarea,l)
	      set y [expr {$y2 - $slope*($x2 - $x)}]
	    }
	  } elseif {($x1 > $State(plotarea,r))} {
	    # And off to right
	    set y $State(plotarea,b)
	    set x [expr {$x2 -($y2-$y) /$slope}]
	    if {($y > $State(plotarea,b))} {
	      #  Ooops passes through right boundary, not top
	      set x $State(plotarea,r)
	      set y [expr {$y2 - $slope*($x2 - $x)}]
	    }
	  } else {
	    # And within Y coords.
	    # Check boundary with left edge
	    set y $State(plotarea,b)
	    set x [expr {$x2 -(($y2-$y) /$slope)}]
	    puts "NEW XY: $x $y $x2 - (($y2-$y)/$slope)"
	  }

	} else {
	  puts "HUH?"
	}
	lappend pts($num) $x $y
	# puts "APPEND: $x $y - $pts($num)"

	# if x2/y2 out of bounds, append new point

	if {($x2 > $State(plotarea,r)) ||($x2 < $State(plotarea,l)) ||
	    ($y2 < $State(plotarea,t)) ||($y2 > $State(plotarea,b))} {
	  # puts "SECOND PT OFF"
	  if {($x2 < $State(plotarea,l))} {
	    # Off left boundary
	    if {($y1 < $State(plotarea,t))} {
	      # And above top
	      # Check boundary with left edge
	      set x $State(plotarea,l)
	      set y [expr {$y1 - $slope*($x1 - $x)}]
	      if {($y < $State(plotarea,t))} {
		#  Ooops passes through top boundary, not left
		set y $State(plotarea,t)
		set x [expr {$x1 -($y1-$y) /$slope}]
	      }
	    } elseif {($y1 > $State(plotarea,b))} {
	      # And below bottom
	      set x $State(plotarea,l)
	      set y [expr {$y1 - $slope*($x1 - $x)}]
	      if {($y > $State(plotarea,b))} {
		#  Ooops passes through bottom boundary, not left
		set y $State(plotarea,b)
		set x [expr {$x1 -($y1-$y) /$slope}]
	      }
	    } else {
	      # And within Y coords.
	      # Check boundary with left edge
	      set x $State(plotarea,l)
	      set y [expr {$y1 - $slope*($x1 - $x)}]
	    }
	  } elseif {($x2 > $State(plotarea,r))} {
	    # puts "X2: NO RIGHT SUPPORT YET"
	  } elseif {($y2 < $State(plotarea,t))} {
	    puts "OFF TOP"
	    # Off top edge

	    if {($x2 < $State(plotarea,l))} {
	      puts "off left"
	      # And off left
	      # Check boundary with top edge
	      set y $State(plotarea,t)
	      set x [expr {$x1 -($y1-$y) /$slope}]
	      if {($x < $State(plotarea,l))} {
		#  Ooops passes through top boundary, not left
		set x $State(plotarea,l)
		set y [expr {$y1 - $slope*($x1 - $x)}]
	      }
	    } elseif {($x2 > $State(plotarea,r))} {
	      puts "off right"
	      # And off to right
	      set y $State(plotarea,t)
	      set x [expr {$x1 -($y1-$y) /$slope}]
	      if {($y > $State(plotarea,b))} {
		#  Ooops passes through right boundary, not top
		set x $State(plotarea,r)
		set y [expr {$y1 - $slope*($x1 - $x)}]
	      }
	    } else {
	      # And within Y coords.
	      # Check boundary with left edge
	      puts "just top"
	      set y $State(plotarea,t)
	      set x [expr {$x1 -($y1-$y) /$slope}]
	    }
	  } elseif {($y2 > $State(plotarea,b))} {
	    # puts "Y2: NO BOTTOM SUPPORT YET"
	  }
	  # puts "NEW XY: $x $y"
	  lappend pts($num) $x $y
	}
      }
    }

    foreach num [array names pts] {
# puts "PTS: $num: $pts($num)"
      if {[llength $pts($num)] >= 4} {
	set id [eval $cvs create line $pts($num) -fill $State($num,color)\
	  -tag [list [list line-$num dset-$num]] -width $State($num,width)\
	  -dash $State($num,dash)]

	foreach indx [array names State bind,line,*] {
	  foreach {bnd ln event} [split $indx ,] {break}
	  if {![string equal {} $State($indx)]} {
	    $cvs bind $id $event "$State($indx) $cvs %x %y  [lindex $pts($num)\
	      0] [lindex $pts($num) 1] $id"
	  }
	}
      }
    }
  }

  proc drawGraph {cvs} {
    upvar [namespace current]::State_$cvs State

    $cvs delete all

    # Check to see if there's some data to plot
    set bad 0
    set ok 0
      
    foreach indx [lsort [array names State  *,x]] \
            indy [lsort [array names State  *,y]]  {
      if {[string first "legend" $indx] >=0} {continue}
      set dset [lindex [split $indx ,] 0]

      if {[string equal $State(style) "pie"]} {
        if {[llength $State($indy)] == 0} {
          set bad $dset
        } else {
          set ok 1
        }
      } else {
        if {([llength $State($indx)] == 0) || ([llength $State($indy)] == 0) ||
            ([llength $State($indx)] != [llength $State($indy)])} {
          set bad $dset
        } else {
          set ok 1
        }   
      }
      if {$bad} {break}
    }

    if {!$ok} {
        if {$bad <= 0} {
          return
        } else {
          error "ERROR IN INPUT (probably missing data)" \
                "Problem in dataset $bad\ Count X: [llength $State($bad,x)] Count y: [llength $State($bad,y)]\n$State($bad,x)\n$State($bad,y)"
        }
    }

    if {[winfo ismapped $cvs]} {
      set border [expr {[$cvs cget -borderwidth] * 2 + 2}]
      set w [expr {[winfo width $cvs] - $border}]
      set h [expr {[winfo height $cvs] - $border}]
    } else {
      set w [$cvs cget -width]
      set h [$cvs cget -height]
    }

    # Override w/h with scrollregion if it's defined
    # Assume 0 based canvases, nothing silly in negative 

    foreach {l t r b} [$cvs cget -scrollregion] {break}
    if {[info exists b]} {
      if {$b > $h} {set h $b}
      if {$r > $w} {set w $r}
    }

    if {$State(style) ne "pie"} {

      set data {}

      foreach d [array names State *\[1-9\],x] {
	foreach {num x} [split $d ,] {break}
	if {(![info exists State($num,selected)]) ||(!$State($num,selected))}\
	  {continue}

	if {$State($num,axis,x,use)} {
	  if {[string equal $State(histStyle) "single"] && $State(rotate)} {
	    if {[llength $data] > 0} {
	      set l1 [llength $State($d)]
	      set l2 [llength $data]
	      foreach v $State($d) v2 [lrange $data [expr {$l2 - $l1}] end] {
		if {[string equal {} $v2]} {set v2 0}
		lappend data [expr {$v + $v2}]
	      }
	    } else {
	      set data $State($d)
	    }
	  } else {
	    eval lappend data $State($d)
	  }
	} else {
	  SetTicks $cvs "x" $State($d) [- $w $State(0,axis,x,LeftMargin)] $num
	}
      }

      if {$data ne ""} {
	set t [time {SetTicks $cvs "x" $data $w}]
	# puts "SetTicks: X: $t "
      }

      set data {}

      foreach d [array names State *\[1-9\],y] {
	foreach {num y} [split $d ,] {break}
	if {(![info exists State($num,selected)]) ||(!$State($num,selected))}\
	  {continue}
	if {$State($num,axis,y,use)} {
	  if {[string equal $State(histStyle) "single"] && !$State(rotate)} {
	    if {[llength $data] > 0} {
	      set l1 [llength $State($d)]
	      set l2 [llength $data]
	      foreach v $State($d) v2 [lrange $data [expr {$l2 - $l1}] end] {
		if {[string equal {} $v2]} {set v2 0}
		lappend data [expr {$v + $v2}]
	      }
	    } else {
	      set data $State($d)
	    }
	  } else {
	    eval lappend data $State($d)
	  }
	} else {
	  SetTicks $cvs y $State($d) $h $num
	}
      }

      if {$data ne ""} {
	set t [time {SetTicks $cvs "y" $data $h}]
	# puts "SetTicks: Y: $t "
      }


      if {$State(0,axis,y,use)} {
	DrawYAxis $cvs
      } else {
	set State(plotarea,t) $State(0,axis,y,TopMargin)
	set State(plotarea,b) [expr $h + $State(0,axis,y,BottomMargin)]
      }

      if {$State(0,axis,x,use)} {
	DrawXAxis $cvs
      } else {
	set State(plotarea,l) $State(0,axis,x,LeftMargin)
	set State(plotarea,r) [expr {$w - $State(0,axis,x,RightMargin)}]
      }

      switch $State(style) {
      scatter {
	  drawScatter $cvs
	}
      line {
	  drawLines $cvs

	  if {$State($num,drawPoint)} {
	    drawScatter $cvs
	  }
	}
      colorbar {
	  DrawColorbar $cvs
	}
      area {
	  DrawArea $cvs
	}
      histogram {
	  drawHistogram $cvs
	}
      default {
	  tk_messageBox -type ok -message "Unknown graph type: $State(style)"
	}
      }

      if {$State(legend,use)} {
	DrawLegend $cvs
      }
    } else {
      DrawPie $cvs
    }
    eval $State(script)
  }

  proc AddBinding {cvs cmd seq idList xList yList} {
    foreach item $idList xo $xList yo $yList {
      $cvs bind $item $seq "$cmd $cvs %x %y $xo $yo $item"
    }
  }

  proc drawScatter {cvs} {

    upvar [namespace current]::State_$cvs State

    foreach nm [lsort -dict [array names State *,x]] {

      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}

      if {![info exists State($num,tagList)]} {
        set State($num,tagList) {}
      }
      
      MakeScaleProcs $cvs $num

      foreach xo $State($num,x) yo $State($num,y) {
        set x [scaleX $xo]
        set y [scaleY $yo]
	lappend ids [DrawPoint $cvs $x $y $State($num,point)\
	  $State($num,color) State($num,tagList) ]
	lappend xs $xo
	lappend ys $yo
      }
    }

    foreach indx [array names State bind,point,*] {
      foreach {bnd ln event} [split $indx ,] {break}
      if {![string equal {} $State($indx)]} {
	AddBinding $cvs $State($indx) $event $ids $xs $ys
      }
    }

  }

  proc StartMinMax {axis cvs x y real_x real_y item} {
    variable TMP
    catch {unset TMP}
    $cvs itemconfigure $item -fill green
    set TMP(start_$axis) [set $axis]
    set TMP(motion_$axis) [set $axis]
    set TMP(real) [set real_$axis]
  }

  proc MoveMinMax {axis cvs x y v1 v2 item} {
    variable TMP

    set dst [expr {[set $axis] - $TMP(motion_$axis)}]
    set TMP(motion_$axis) [set $axis]

    if {[string equal $axis "x"]} {
      $cvs move $item $dst 0
    } else {
      $cvs move $item 0 $dst
    }
  }

  proc EndMinMax {axis cvs x y v1 v2 item} {
    variable TMP
    upvar [namespace current]::State_$cvs State

    switch $axis {
    "x" {
	set sm $State(plotarea,l)
	set lg $State(plotarea,r)

	set axMin Min
	set axMax Max
	set tckmin minTick
	set tckmax maxTick
	set op +
      }
    "y" {
	set sm $State(plotarea,t)
	set lg $State(plotarea,b)

	set axMin Max
	set axMax Min
	set tckmin maxTick
	set tckmax minTick
	set op -
      }
    }

    set mid [expr {($lg+$sm) / 2.0}]

    if {$TMP(start_$axis) < $mid} {
      set posit $axMin
      set mod2 0,axis,$axis,$tckmin
    } else {
      set mod2 0,axis,$axis,$tckmax
      set posit $axMax
    }

    foreach in [array names State *,axis,$axis,use] {
      if {$State($in)} {
	set mod [string map [list use $posit] $in]
	if {[string equal {} $State($mod)]} {
	  set State($mod) $State($mod2)
	}
      }
    }

    set pixelPerStep [expr {($State(0,axis,$axis,maxTick)\
      - $State(0,axis,$axis,minTick)) /($lg - $sm)}]

    set modDelta [expr {$pixelPerStep *($TMP(start_$axis)\
      - $TMP(motion_$axis))}]

    foreach in [array names State *,axis,$axis,use] {
      if {$State($in)} {
	set mod [string map [list use $posit] $in]
	set State($mod) [expr $State($mod) $op $modDelta]
      }
    }
    drawGraph $cvs
  }

  proc PopupByPlotArea {cvs x y v1 v2 item} {
    upvar [namespace current]::State_$cvs State

    set px1 $State(plotarea,l)
    set px2 $State(plotarea,r)
    set rx1 $State(0,axis,x,minTick)
    set rx2 $State(0,axis,x,maxTick)
    set fract [expr {double($x - $px1) /($px2-$px1)}]
    set xp [expr {$rx1 +(($rx2 - $rx1) * $fract)}]

    # puts "$px1 $px2 $rx1 $rx2 $x"
    # puts "XVAL: $xp"

    set py1 $State(plotarea,t)
    set py2 $State(plotarea,b)
    set ry1 $State(0,axis,y,minTick)
    set ry2 $State(0,axis,y,maxTick)
    set fract [expr {double($y - $py1) /($py2-$py1)}]
    set yp [expr {$ry2 -(($ry2 - $ry1) * $fract)}]

    # puts "$py1 $py2 $ry1 $ry2 $y"
    # puts "yVAL: $yp"

    set it [$cvs create text [+ 2 $x] [- $y 6] -anchor sw -text "$xp $yp"]
    $cvs bind $it <Motion> "$cvs delete $it"
    $cvs bind $item <Motion> "$cvs delete $it"
  }

  proc Popup {cvs x y v1 v2 item} {
    if {$x < 20} {incr x 35}
    if {$y < 20} {incr y 35}

    set it [$cvs create text [+ 2 $x] [- $y 6] -anchor sw -text "$v1 $v2"]
    $cvs bind $it <Motion> "$cvs delete $it"
    $cvs bind $item <Motion> "$cvs delete $it"
  }

  proc DrawPie {cvs} {
    upvar [namespace current]::State_$cvs State
    
    # Used in placing pies and titles
    set fontHt [font metric $State(legend,font) -linespace]

    set datasetCount [llength [array names State *,x]]
    # The  0,y index is empty
    incr datasetCount -1

    set ht [winfo height $cvs]
    set wd [winfo width $cvs]
    if {$wd > $ht} {
      set radius $ht
    } else {
      set radius $wd
    }
    set radius [expr {$radius * .4}]
    set divisor 1
    
    if {$datasetCount > 1} {
      set divisor [expr {int(sqrt($datasetCount-1)) + 1}]
      set radius [expr {$radius / $divisor}]
    }
    set steps [expr $radius * 2.2]

    set baseX 2
    set baseY 2

    set sum 0
    set colorCount [llength $State(colors)]
    set numList {}

    # Is there a reason for multiple pies on a single canvas?
    # Start for support, but not really using it.
    # How to handle multiple legends?

    foreach indx [array names State *,x] indy [array names State *,y] {
      foreach {dataset x} [split $indx ","] {break}
      if {[string equal "legend" $dataset]} {continue}
      if {![string equal "" $State($dataset,legend,title)]} {
	lappend pointList {}
	lappend colorList {}
	lappend txtList $State($dataset,legend,title)
	lappend numList $dataset
      }

      set id 0
      set sliceTotal 0
      foreach slicesize $State($indy) {
	set sliceTotal [expr {double($sliceTotal + $slicesize)}]
      }

      foreach slicesize $State($indy) txt $State($indx) {
	set color [lindex $State(colors) [expr {$id % $colorCount}]]
	lappend txtList $txt
	lappend colorList $color
	lappend pointList filled-bigbox

	lappend slices [list $txt [expr {$slicesize/$sliceTotal}] $color]
	incr id
      }
      
      set row [expr {(($dataset-1) / $divisor)}]
      set txtoff [expr {$fontHt * $row}]
      set yp [expr {$baseY + ($row * $steps) + ($row * $txtoff)}]
      set yt [expr $yp + $steps + $fontHt]

      set xp [expr {$baseX + ((($dataset-1) % $divisor) * $steps)}]
#      set yp [expr {$baseY + ($row * $steps) + ($row * $txtoff)}]
      $cvs create text [expr {$xp + ($steps/2)}]  $yt \
          -text $State($dataset,legend,title) -anchor s \
	  -font "$State(legend,font) bold"
      PieChartDrawPie $cvs pieChart $xp $yp $radius $slices
      DrawLegendKernel $cvs $txtList $pointList $colorList $numList
    }
  }

  proc interpolateCoord {xval coords} {
    if {[llength $coords] <= 4} {return 0}
    set yprev 0
    foreach {x1 y1} $coords {x2 y2} [lrange $coords 2 end] {
      if {($x1 < $xval) &&($x2 >= $xval) ||($x1 <= $xval) &&($x2 > $xval)} {
	set yprev [expr {((($xval - $x1) *($y2 - $y1)) /($x2 - $x1)) + $y1}]
	return $yprev
      }
      if {$x1 > $xval} {break}
      set yprev $y1
    }
    return 0
    return $yprev
  }


  proc DrawArea {cvs} {
    upvar [namespace current]::State_$cvs State
parray State

    set idList {}

    if {[winfo ismapped $cvs]} {
      set border [expr {[$cvs cget -borderwidth] * 2 + 2}]
      set w [expr {[winfo width $cvs] - $border}]
      set h [expr {[winfo height $cvs] - $border}]
      # set h [winfo height $cvs]
    } else {
      set h [$cvs cget -height]
    }
    set b [- $h $State(0,axis,y,BottomMargin)]

    set yMax $b
    set ppy $b

    set prevCoords {}

    foreach nm [lsort -dict [array names State *,x]] {

      foreach {num s x} [split $nm ,] {break}

      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}


      MakeScaleProcs $cvs $num

      foreach x $State($num,x) y $State($num,y) {
        set x1 [scaleX $x] 
	lappend coords $x1 [scaleY $y]
      }
      
      if {(![info exists coords]) || ([llength $coords] == 0)} {continue}

      set color $State($num,color)
      set use [list [lindex $coords 0] $b]
      lappend use [lindex $coords 0] [lindex $coords 1]
      catch {unset ymod}
      set xpos 0
      set xfirst [lindex $coords 0]
      set prevLen [llength $prevCoords]
      while {([lindex $prevCoords $xpos] < $xfirst) &&($xpos < $prevLen)} {
	incr xpos 2
      }

      foreach {xp yp} $coords {
	if {$xp <= [lindex $prevCoords $xpos] ||($prevLen == 0)} {
	  set ymod [interpolateCoord $xp $prevCoords]
	  if {$ymod > 0} {
	    set ymod [expr {$b - $ymod}]
	  }
	  set yu [expr {$yp - $ymod}]
	  lappend use $xp $yu
	} else {
	  if {$xpos == 0} {
	    set xu [lindex $prevCoords $xpos]
	    set yt [interpolateCoord $xu $coords]
	    lappend use $xu $yt

	  }
	  while {([lindex $prevCoords $xpos] < $xp) &&($xpos < $prevLen)} {
	    set xu [lindex $prevCoords $xpos]
	    incr xpos
	    set ymod [expr {$b - [lindex $prevCoords $xpos]}]
	    set yt [interpolateCoord $xu $coords]
	    set yu [expr {$yt - $ymod}]
	    lappend use $xu $yu
	    incr xpos
	  }
	  if {$xpos >= [llength $prevCoords]} {
	    set xu [lindex $prevCoords end-1]
	    set yt [interpolateCoord $xu $coords]
	    lappend use $xu $yt
	  }
	  set ymod [interpolateCoord $xp $prevCoords]
	  if {$ymod > 0} {
	    set ymod [expr {$b - $ymod}]
	  }
	  set yu [expr {$yp - $ymod}]
	  lappend use $xp $yu
	}
      }

      # Drop line back to origin or intersect prev graphic.

      lappend use $x1 $b

      set idList "[eval $cvs create polygon $use -fill $color -outline $color\
	-width 0] $idList"
      set prevCoords [lrange $use 2 end-2]
      set coords {}
    }
    #     DrawLegendKernel $cvs $txtList $pointList $colorList $numList
    foreach id $idList {
      $cvs raise $id
    }
  }


  proc OLDDrawColorbar {cvs} {
    upvar [namespace current]::State_$cvs State

    # Only colors that have been used will be displayed in the
    # legend.  Wipe them here, and set State($colorPosition,colorUbsed)
    # as those colors are selected.

    foreach ind [array names State *,colorUsed] {
      unset State($ind)
    }

    set prevx {}
    if {[winfo ismapped $cvs]} {
      set border [expr {[$cvs cget -borderwidth] * 2 + 2}]
      set w [expr {[winfo width $cvs] - $border}]
      set h [expr {[winfo height $cvs] - $border}]
    } else {
      set h [$cvs cget -height]
      set w [$cvs cget -width]
    }
    set b [- $h $State(0,axis,y,BottomMargin)]
    set r [- $w $State(0,axis,x,RightMargin)]
    set l $State(0,axis,x,LeftMargin)

    foreach in [array names State *,z] {
      foreach z $State($in) {
	if {![info exists zmax]} {
	  set zmax $z
	  set zmin $z
	}
	if {$z > $zmax} {set zmax $z}
	if {$z < $zmin} {set zmin $z}
	set hits($z) 1
      }
    }

    if {![string equal "" $State(color,min)]} {
      set zmin $State(color,min)
    }

    set colorCount [llength $State(colors)]
    set divisor [expr {double($zmax - $zmin) /($colorCount-1)}]

    if {$State(rotate)} {
      set ybot $State(plotarea,l)
    } else {
      set ybot $State(plotarea,b)
    }

    set State(colorbar,zmin) $zmin
    set State(colorbar,divisor) $divisor

    foreach nm [lsort -dict [array names State *,x]] {

      foreach {num s x} [split $nm ,] {break}

      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}

      set xa $ybot
      set ya $ybot

      set zpos($num) 0

      MakeScaleProcs $cvs $num

      foreach xo $State($num,x) yo $State($num,y) {
        set x1 [scaleX $xo]
        set y1 [scaleY $yo]

	set z [lindex $State($num,z) $zpos($num)]
	incr zpos($num)

	set colorPos [expr {int(($z-$zmin) /$divisor)}]
	set color [lindex $State(colors) $colorPos]

	set State($colorPos,colorUsed) 1
	set State($colorPos,selected) 1

	if {$State(rotate)} {
	  set item [DrawBar $cvs $xa $y1 $x1 [- $y1 $State($num,width)] $color \
	      $State(rotate) $num]
	  set xa $x1
	  set nprev $num
	} else {
	  set item [DrawBar $cvs $x1 $ya [+ $x1 $State($num,width)] $y1 $color \
	      $State(rotate) $num]
	  set ya $y1
	  set nprev $num
	}

	foreach indx [array names State bind,bar,*] {
	  foreach {bnd ln event} [split $indx ,] {break}
	  if {![string equal {} $State($indx)]} {
	    $cvs bind $item $event "$State($indx) $cvs %x %y $yo $z $item"
	  }
	}
      }
    }
  }

proc DrawColorbar {cvs} {
    upvar [namespace current]::State_$cvs State

    # Only colors that have been used will be displayed in the
    # legend.  Wipe them here, and set State($colorPosition,colorUsed)
    # as those colors are selected.

    foreach ind [array names State *,colorUsed] {
      unset State($ind)
    }

    foreach in [array names State *,z] {
      foreach z $State($in) {
	if {![info exists zmax]} {
	  set zmax $z
	  set zmin $z
	}
	if {$z > $zmax} {set zmax $z}
	if {$z < $zmin} {set zmin $z}
	set hits($z) 1
      }
    }

    if {![string equal "" $State(color,min)]} {
      set zmin $State(color,min)
    }

    set colorCount [llength $State(colors)]
    set divisor [expr {double($zmax - $zmin) /($colorCount-1)}]

    set State(colorbar,zmin) $zmin
    set State(colorbar,divisor) $divisor

    # A bar has a major and minor dimension.
    #  For non-rotated, Major is Y and minor is X
    #  For rotated, Major is X and minor is Y

    if {$State(rotate)} {
       set majorBase $State(plotarea,l)
       set minor y
       set major x
       incr majorBase 
    } else {
       set majorBase $State(plotarea,b)
       set minor x
       set major y
    }

       
    # Find out how many bars exist at each minor location.
    # This will be X axis if not rotated, or Y if rotated.
    # This will be used to set the minor-dimension offsets.

    foreach nm [lsort -dict [array names State *,$minor]] {
      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}
      foreach b $State($nm) {
        if {![info exists inUse($b)]} {
          set inUse($b) 0
	  set majorEnd($b) $State(0,axis,$major,minTick)
	  set axisStep($b) $State(0,axis,$major,minTick)
        }
        incr inUse($b)
      }
    }

    foreach nm [lsort -dict [array names State *,x]] {
      
      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}

puts "drawColorbar: $nm $State($nm) "

      MakeScaleProcs $cvs $num

      foreach xo $State($num,x) yo $State($num,y) z $State($num,z) {
          set x [scaleX $xo]
          set y [scaleY $yo]
# puts "$num XO: $xo YO: $yo X: $x Y: $y"
	  set minorOffset [expr {$inUse([set ${minor}o])*.5}]
	  incr inUse([set ${minor}o]) -2

	  set colorPos [expr {int(($z-$zmin) /$divisor)}]
	  set color [lindex $State(colors) $colorPos]

	  set State($colorPos,colorUsed) 1
	  set State($colorPos,selected) 1

          if $State(rotate) {
	      set min1 [expr {$y + (.5*$State($num,width))}]
	      set maj1 [scaleX $majorEnd($yo)]
	      set x2 [expr $majorEnd($yo) + $xo - $axisStep($yo)]
	      set maj2 [scaleX $x2]
	      set majorEnd($yo) $x2
	      set axisStep($yo) 0

  	      set min2 [expr {$min1 - $State($num,width)}]
	      lappend items [DrawBar $cvs $maj1 $min1 $maj2 $min2 \
	        $color $State(rotate) $num bar-$xo $xo]
              lappend xs $y
              lappend ys $x
          } else {
	      set min1 [expr {$x - ($State($num,width) *.5)}]
	      set maj1 [scaleY $majorEnd($xo)]
	      set y2 [expr $majorEnd($xo) + $yo - $axisStep($xo)]
	      set maj2 [scaleY $y2]
	      set majorEnd($xo) $y2
	      set axisStep($xo) 0
	    set min2 [expr {$min1 + $State($num,width)}]
	    lappend items [DrawBar $cvs $min1 $maj1 $min2 $maj2 \
	        $color $State(rotate) $num bar-$xo $yo]
            lappend xs $x
            lappend ys $y
          }
      }
  }
	  
  foreach indx [array names State bind,bar,*] {
    foreach {bnd ln event} [split $indx ,] {break}
    if {![string equal {} $State($indx)]} {
      AddBinding $cvs $State($indx) $event $items $xs $ys
    }
  }
}

proc drawHistogram {cvs} {
    upvar [namespace current]::State_$cvs State
    
    # A bar has a major and minor dimension.
    #  For non-rotated, Major is Y and minor is X
    #  For rotated, Major is X and minor is Y

    if {$State(rotate)} {
       set majorBase $State(plotarea,l)
       set minor y
       set major x
       incr majorBase 
    } else {
       set majorBase $State(plotarea,b)
       set minor x
       set major y
    }

       
    # Find out how many bars exist at each minor location.
    # This will be X axis if not rotated, or Y if rotated.
    # This will be used to set the minor-dimension offsets.

    foreach nm [lsort -dict [array names State *,$minor]] {
      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}
      foreach b $State($nm) {
        if {![info exists inUse($b)]} {
          set inUse($b) 0
	  set majorEnd($b) $State(0,axis,$major,minTick)
	  set axisStep($b) $State(0,axis,$major,minTick)
        }
        incr inUse($b)
      }
    }

    foreach nm [lsort -dict [array names State *,x]] {
      
      foreach {num x} [split $nm ,] {break}
      if {![string is double $num]} {continue}
      if {!$State($num,selected)} {continue}

# puts "drawHist: $nm $State($nm) "

      MakeScaleProcs $cvs $num

      foreach xo $State($num,x) yo $State($num,y) {
          set x [scaleX $xo]
          set y [scaleY $yo]
# puts "$num XO: $xo YO: $yo"
	  set minorOffset [expr {$inUse([set ${minor}o])*.5}]
	  incr inUse([set ${minor}o]) -2

          if $State(rotate) {
	    if {[string match stacked $State(histStyle)]} {
	      set min1 [expr {$y + ($minorOffset*$State($num,width))}]
	      set maj1 $majorBase
	      set maj2 $x
	    } else {
	      set min1 [expr {$y + (.5*$State($num,width))}]
	      set maj1 [scaleX $majorEnd($yo)]
	      set x2 [expr $majorEnd($yo) + $xo - $axisStep($yo)]
	      set maj2 [scaleX $x2]
	      set majorEnd($yo) $x2
	      set axisStep($yo) 0
	    }
  	    set min2 [expr {$min1 - $State($num,width)}]
	    lappend items [DrawBar $cvs $maj1 $min1 $maj2 $min2 \
	        $State($num,color) $State(rotate) $num bar-$xo $xo]
            lappend xs $y
            lappend ys $x
          } else {
	    if {[string match stacked $State(histStyle)]} {
	      set min1 [expr {$x - ($State($num,width) * $minorOffset)}]
	      set maj1 $majorBase
	      set maj2 $y
	    } else {
	      set min1 [expr {$x - ($State($num,width) *.5)}]
	      set maj1 [scaleY $majorEnd($xo)]
	      set y2 [expr $majorEnd($xo) + $yo - $axisStep($xo)]
	      set maj2 [scaleY $y2]
	      set majorEnd($xo) $y2
	      set axisStep($xo) 0
	    }
	    set min2 [expr {$min1 + $State($num,width)}]
	    lappend items [DrawBar $cvs $min1 $maj1 $min2 $maj2 \
	        $State($num,color) $State(rotate) $num bar-$xo $yo]
            lappend xs $x
            lappend ys $y
          }
      }
  }
	  
  foreach indx [array names State bind,bar,*] {
    foreach {bnd ln event} [split $indx ,] {break}
    if {![string equal {} $State($indx)]} {
      AddBinding $cvs $State($indx) $event $items $xs $ys
    }
  }
}

  proc Insert {lst new} {
    set loc 0
    set hit 0
    foreach el $lst {
      if {$el > $new} {
	set lst [linsert $lst $loc $new]
	set hit 1
	break
      }
      incr loc
    }
    if {!$hit} {lappend lst $new}
    return $lst
  }


  proc DrawBar {cvs corn1x corn1y corn2x corn2y color rotate num {tag bar} {value {}}} {
    upvar [namespace current]::State_$cvs State
# puts "cvs $cvs corn1x $corn1x corn1y $corn1y corn2x $corn2x corn2y $corn2y color $color rotate $rotate tag $tag value  $value"

    upvar [namespace current]::State_$cvs State

    if {$corn1x == $corn2x} {return -1}
    if {$corn1y == $corn2y} {return -1}
    
    set item [$cvs create rectangle $corn1x $corn1y $corn2x $corn2y \
	-fill $color -outline $color -tag $tag -width 0]

    if {$State(histShowAbsolute)} {
      if {$rotate} {
          set ty [expr {int(($corn2y + $corn1y)/2)-2}]
  	  set tx [expr {$corn1x>$corn2x?$corn1x:$corn2x}]
	  set anchor w
        if {[string equal stacked $State(histStyle)]} {
	  incr tx 3
	} else {
	  incr ty -$State($num,width)
	  incr tx -3
	}
      } else {
        set ty [expr {$corn1y<$corn2y?$corn1y:$corn2y}]
	set tx [expr {int(($corn2x + $corn1x)/2)}]
	set anchor s
        if {[string equal stacked $State(histStyle)]} {
  	  incr ty -3
	} else {
	  incr tx $State($num,width)
	  incr ty 3
	}
      }
       
      $cvs create text $tx $ty -text $value\
	  -font $State(axis,y,font) -anchor $anchor
    }
    return $item
  }

  proc DrawLegend {maincvs} {
    upvar [namespace current]::State_$maincvs State
    
    set txtList {}
    set numList {}
    set pointList {}
    set colorList {}

    if {[string equal "colorbar" $State(style)]} {
      if {![string equal "" $State(0,legend,title)]} {
	set pointList {{}}
	set colorList {{}}
	lappend txtList $State(0,legend,title)
	lappend numList 0
      }

      for {set i 0} {$i < [llength $State(colors)]} {incr i} {
#	set num [expr {$i*$State(colorbar,divisor)-$State(colorbar,zmin)}]
	set num [expr {$i*$State(colorbar,divisor)+$State(colorbar,zmin)}]
	lappend txtList [format "%.3f" $num]
	lappend colorList [lindex $State(colors) $i]
	lappend numList $i
	lappend pointList "filled-bigbox"
      }
    } else {
      foreach ind [lsort [array names State *,legend,title]] {
	if {[string equal "" $State($ind)]} {continue}
	foreach {num l t} [split $ind ,] {break}
	lappend txtList $State($ind)
	lappend colorList $State($num,color)
	lappend numList $num
	if {$num == 0} {
	  lappend pointList {}
	} else {
	  lappend pointList $State($num,point)
	}
      }
    }
    DrawLegendKernel $maincvs $txtList $pointList $colorList $numList
  }

  proc DrawLegendKernel {maincvs txtList pointList colorList numList} {
# puts "maincvs $maincvs txtList $txtList pointList $pointList colorList $colorList numList $numList"
    upvar [namespace current]::State_$maincvs State

    catch {destroy $maincvs.legend}
    set cvs [canvas $maincvs.legend -background [$maincvs cget -background]\
      -relief raised -borderwidth 2]

    $cvs xview moveto 0
    $cvs yview moveto 0

    set y 10

    foreach txt $txtList point $pointList color $colorList num $numList {
      if {[string equal "" $txt]} {continue}
      if {[string equal "" $num]} {set num 0}

      set font $State(legend,font)

      if {[string equal "" $point]} {
	foreach {name size} $font {break}
	incr size 2
	set font [list $name $size bold]
      }

      set lineHt [font metrics $font -linespace]
      set lh2 [expr {$lineHt/2}]

      DrawPoint $cvs 5 [+ $y $lh2] $point $color {legend-text legend}

      # Skip elements that don't have selectable option.
      #  The legend for a colorbar reflects the color values, not the
      #  datasets

      if {[string equal colorbar $State(style)]} {
	if {![info exists State($num,colorUsed)]} {continue}
      } else {
	if {![info exists State($num,selected)]} {continue}
      }

      # Yanked from legend text - looks messy to my eye: -fill $color 
      if {$State($num,selected)} {
	set txtColor black
      } else {
	set txtColor gray75
      }

      set id [$cvs create text 15 $y -text $txt -font $font -anchor nw\
	-tags {legend-text legend} -fill $txtColor]
      incr y $lineHt
      if {$State($num,selected)} {
	$cvs bind $id <Button-1> "[namespace current]::OnOffDataset $maincvs\
	  $num 0"
      } else {
	$cvs bind $id <Button-1> "[namespace current]::OnOffDataset $maincvs\
	  $num 1"
      }
    }
    foreach {l t r b} [$cvs bbox all] {break}

    if {![info exists l]} {
      destroy $cvs
      return

    }

    set w [expr {$r - $l + 10}]
    set h [expr {$b - $t + 10}]
    incr l -5
    incr t -5

    $cvs configure -height $h -width $w -scrollregion [list $l $t $r $b]

    set lx $State(legend,x)
    set ly $State(legend,y)

    if {![string equal "" $State(legend,relx)]} {
      set lx [expr {$State(legend,relx) * [winfo width $maincvs]}]
    }
    if {![string equal "" $State(legend,rely)]} {
      set ly [expr {$State(legend,rely) * [winfo height $maincvs]}]
    }

    if {![string equal "" $State(legend,X)]} {
      set lx $State(legend,X)
    }
    if {![string equal "" $State(legend,Y)]} {
      set ly $State(legend,Y)
    }

    set legend [$maincvs create window $lx $ly -window $cvs\
      -anchor $State(legend,anchor) -tag legend]

    bind $maincvs <Double-1> "[namespace current]::toggleLegend $maincvs"
    bind $cvs <Double-1> "[namespace current]::toggleLegend $maincvs"
    $maincvs raise $legend

    #     bind $cvs <ButtonPress-1> \
#      "$maincvs raise $legend; [namespace current]::SetLocation %X %Y"
    bind $cvs <ButtonRelease-1> "$maincvs raise $legend;\
      [namespace current]::ClearLocation"
    bind $cvs <B1-Motion> "[namespace current]::MoveItem $maincvs $legend %X %Y"

  }

  proc OnOffDataset {cvs num new} {
    upvar [namespace current]::State_$cvs State

    set State($num,selected) $new
    drawGraph $cvs
  }

  proc toggleLegend {cvs} {
    set w [winfo width $cvs]
    foreach {l t r b} [$cvs bbox legend] {break}

    if {($l > 0) &&($l < $w)} {
      $cvs move legend -2000 -2000
    } else {
      $cvs move legend 2000 2000
    }
  }
  
################################################################
#   proc clearGraph {cvs}--
#    Remove graph data
# Arguments
#   cvs - the canvas identifier for a graph
# 
# Results
#   Deletes all data associated with this graph.
# 
  proc clearGraph {cvs} {
    upvar [namespace current]::State_$cvs State
    unset State
  }

  ################################################################
  #   proc squareAtChange {xs ys}--
  #    Converts a dataset from angled lines to squared off
  #    lines by inserting new cusp points where the previous cusps
  #    occurred
  #
  # Arguments
  #   xs 	X Coords
  #   ys  Y Coords
  # Results
  #   Returns 2 new lists of coordinates.
  # 
  proc squareAtChange {xs ys} {
    set py [lindex $ys 0]

    set newx [lindex $xs 0]
    set newy $py

    foreach x [lrange $xs 1 end] y [lrange $ys 1 end] {
      lappend newx $x
      lappend newy $py
      lappend newx $x
      lappend newy $y
      set py $y
    }
    return [list $newx $newy]
  }

  ################################################################
  #   proc squareAtMidPt {xs ys}--
  #    Converts a dataset from angled lines to squared off
  #    lines by inserting new cusp points at the midpoints of
  #    previous cusps
  # Arguments
  #   xs 	X Coords
  #   ys  Y Coords
  # Results
  #   Returns 2 new lists of coordinates.
  # 
  proc squareAtMidPt {xs ys} {

    set py [lindex $ys 0]
    set newx [lindex $xs 0]
    set newy $py

    foreach x1 [lrange $xs 0 end-1] x2 [lrange $xs 1 end] y1 [lrange $ys 0\
      end-1] y2 [lrange $ys 1 end] {
      set nx [expr {($x1 + $x2) /2.0}]
      lappend newx $nx
      lappend newy $y1
      lappend newx $nx
      lappend newy $y2
    }
    lappend newx $x2
    lappend newy $y2
    return [list $newx $newy]
  }
}

if {[info exists argv] &&([set p [lsearch $argv "-tktest"]] >= 0) ||\
  [info exists argv] &&($argv == "-test")} {
  tk appname "graph2"

  if {$p >= 0} {
    set TESTING 1
    set p2 $p
    incr p2
    set TkTestAddress [lindex $argv $p2]
    if {[string first "." $TkTestAddress] < 0} {
      puts "-test IP.Address"
      exit
    }
    set argv [lreplace $argv $p $p2]
    unset p p2

    source socksend.tcl
    sockappsetup tktest.tcl 3010 $TkTestAddress

    # INVOKE tkreplay as:
    # wish tkreplay.tcl 

    proc ::tk_getOpenFile {args} {
      if {$::tk_strictMotif} {
	return [eval tk::MotifFDialog open $args]
      } else {
	return [eval ::tk::dialog::file:: open $args]
      }
    }

    proc ::tk_getSaveFile {args} {
      if {$::tk_strictMotif} {
	return [eval tk::MotifFDialog save $args]
      } else {
	return [eval ::tk::dialog::file:: save $args]
      }
    }

  }

  proc + {a b} {
    return [expr {$a + $b}]
  }
  proc - {a b} {
    return [expr {$a - $b}]
  }
  proc * {a b} {
    return [expr {$a * $b}]
  }


  set rangeTests {
    2 7 {0.5 1.5 7.25}             20 70 {5.0 15.0 72.5}
    -20 70 {10.0 -30.0 75.0}      -.2 .7 {0.1 -0.3 0.7}
    -.02 .07 {0.01 -0.03 0.075}   -25 50 {10.0 -30.0 55.0}
    25 30 {0.5 24.5 30.25}        25 40 {2.0 24.0 41.0}
    25 50 {5.0 20.0 52.5}         .25 .3 {0.005 0.245 0.3025}
    .25 .4 {0.02 0.24 0.4}        .25 .5 {0.05 0.2 0.525}
    0 1 {0.1 0.0 1.05}            0 2 {0.2 0.0 2.1}
    0 5 {0.5 0.0 5.25}            0 10 {1.0 0.0 10.5}
    26 54 {5.0 25.0 55.0}         25 41 {2.0 24.0 42.0}
    -54 -26 {5.0 -55.0 -25.0}    -41 -25 {2.0 -42.0 -24.0}
    0 20 {2.0 0.0 21.0}           0 10 {1.0 0.0 10.5}
}
  puts "Tick Size Tests"
  foreach {min max rtn} $rangeTests {
    if {[set tst [graph2::getTickSize $min $max]] ne $rtn} {
      puts "FAIL: ($min - $max) Returned $tst not $rtn"
    }
  }

  canvas .c -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c -row 0 -column 0 -sticky news
  graph2::create .c -axis,y,lColor red -axis,y,color blue -axis,x,lColor black\
    -axis,x,color orange
  graph2::insertData .c {1 2 3 4 5 6 7} {1 3 5 3 2 1 4} -legend,title "ABC"

  canvas .c2 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c2 -row 0 -column 1 -sticky news
  graph2::create .c2 -axis,y,dash {1 3} -axis,x,dash {3 2} -style line\
    -0,axis,x,LeftMargin 60

  graph2::insertData .c2 {0 7 18 20 22 26 } {0 220 555 610 670 780}\
    -legend,title "Point vs Size" -color red -dataset 1

  graph2::insertData .c2 {1 2 3 4 5 6 7} {3 5 3 2 -1 -4 -2} -dataset 2\
    -color blue -point o -legend,title "SECOND"

  foreach {x1 y1} [graph2::squareAtMidPt {0 7 18 20 22 26 } {0 220 555 610 670\
    780}] {
    break
  }
  foreach {x2 y2} [graph2::squareAtChange {0 7 18 20 22 26 } {0 220 555 610\
    670 780}] {
    break
  }

  graph2::insertData .c2 $x1 $y1 -width 2 -dataset 3 -color orange -point box\
    -legend,title "MidPt"

  graph2::insertData .c2 $x2 $y2 -width 5 -dataset 4 -color yellow -point box\
    -legend,title "Change"

  canvas .c3 -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .c3 -row 0 -column 2 -sticky news
  graph2::create .c3 -axis,y,dash {1 3} -axis,x,dash {3 2}\
    -legend,title "Should overlap" -0,axis,y,BottomMargin 60\
    -0,axis,x,LeftMargin 60
  graph2::insertData .c3 {1 2 3 4 5 6 7} {1 3 5 3 2 -1 -4} -point o -color red
  graph2::insertData .c3 {1 2 3 4 5 6 7} {10 30 50 30 20 -10 -40} -dataset 2\
    -color blue -point x -axis,y,use 0

  canvas .c4 -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .c4 -row 1 -column 0 -sticky news
  graph2::create .c4 -axis,y,dash {1 3} -axis,x,dash {3 2} -axis,y,use 0\
    -axis,x,use 0

  graph2::insertData .c4 {1 2 3 4 5 6 7} {1 3 5 3 2 -1 -4}\
    -legend,title "Overlap" -point filled-o -color red
  graph2::insertData .c4 {1 2 3 4 5 6 7} {10 30 50 30 20 -10 -40} -dataset 2\
    -color black -point x -legend,title "Unscaled"
  #   graph2::drawGraph .c4

  canvas .c5 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c5 -row 1 -column 1 -sticky news
  graph2::create .c5 -axis,y,dash {1 3} -axis,x,dash {3 2} -axis,y,use 0\
    -axis,x,use 0
  graph2::insertData .c5 {1 2 3 4 5 6 7} {1 3 5 3 2 -1 -4} -legend,title "Ace"
  graph2::insertData .c5 {1 2 3 4 5 6 7 8 9} {10 30 50 30 20 -10 -40 30 40}\
    -dataset 2 -color red -point filled-o -legend,title "King"
  #   graph2::drawGraph .c5

  canvas .c6 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c6 -row 1 -column 2
  graph2::create .c6 -axis,y,dash {1 3} -axis,x,dash {3 2} -axis,y,use 0\
    -legend,title "First 7 match"
  graph2::insertData .c6 {1 2 3 4 5 6 7} {1 3 5 3 2 -1 -4} -legend,title "Plus"
  graph2::insertData .c6 {1 2 3 4 5 6 7 8 9} {10 30 50 30 20 -10 -40 30 40}\
    -dataset 2 -color red -point x -legend,title "Ex"
  #   graph2::drawGraph .c6

  grid rowconfigure . 1 -weight 1
  grid columnconfigure . 1 -weight 1

  canvas .c7 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c7 -row 2 -column 0
  graph2::create .c7 -axis,y,dash {1 3} -axis,x,dash {3 2}
  graph2::insertData .c7 {0 0 0 0} {0 0 0 0} -legend,title "Zeros"
  #   graph2::drawGraph .c7

  grid rowconfigure . 1 -weight 1
  grid columnconfigure . 1 -weight 1

  proc smile {cvs x y} {
    $cvs create oval [- $x 5] [- $y 5] [+ $x 5] [+ $y 5] -fill yellow

    $cvs create oval [- $x 3] [- $y 1] [- $x 2] [- $y 2] -fill yellow
    $cvs create oval [+ $x 3] [- $y 1] [+ $x 2] [- $y 2] -fill yellow

    $cvs create arc [- $x 2] [+ $y 3] [+ $x 2] [+ $y 1] -start 230 -extent 185\
      -style arc
  }

  proc frown {cvs x y} {
    $cvs create oval [- $x 5] [- $y 5] [+ $x 5] [+ $y 5] -fill pink

    $cvs create oval [- $x 3] [- $y 1] [- $x 2] [- $y 2] -fill pink
    $cvs create oval [+ $x 3] [- $y 1] [+ $x 2] [- $y 2] -fill pink

    $cvs create arc [- $x 3] [+ $y 3] [+ $x 3] [+ $y 5] -start 45 -extent 180\
      -style arc
  }

  canvas .c8 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c8 -row 2 -column 1
  graph2::create .c8 -axis,y,dash {1 3} -axis,x,dash {3 2}\
    -0,axis,y,BottomMargin 40 -0,axis,y,TopMargin 30 -0,axis,x,LeftMargin 50\
    -0,axis,x,RightMargin 20

  graph2::insertData .c8 {1 2 3 4} {1 3 5 3} -point smile -legend,title "Smiley"
  graph2::insertData .c8 {1 2 3 4} {2 3 4 5} -dataset 2 -point frown\
    -legend,title "Frowney"
  #   graph2::drawGraph .c8

  grid rowconfigure . 1 -weight 1
  grid columnconfigure . 1 -weight 1

  canvas .c9 -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .c9 -row 2 -column 2
  graph2::create .c9 -style histogram -wid 4 -0,axis,y,BottomMargin 40\
    -0,axis,y,TopMargin 30 -0,axis,x,LeftMargin 50 -0,axis,x,RightMargin 20
  graph2::insertData .c9 {1 2 3 4} {1 2 3 4} -color red -legend,title "Red"
  graph2::insertData .c9 {1 2 3 4} {5 4 3 2} -color blue -dataset 2\
    -legend,title "Blue"
  graph2::insertData .c9 {1 2 3 4} {2 2 3 3} -color green -dataset 3\
    -legend,title "Green"

  proc xLabel {lst num} {
    if {[expr {int($num)}] != $num} {
      return
    }
    return [lindex $lst [expr {int($num)}]]
  }

  canvas .cb -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .cb -row 5 -column 1
  graph2::create .cb -style histogram -wid 4 -axis,x,nameProc {xLabel {{} one\
    two three four}} -axis,y,nameProc {xLabel {{} one two three four five six}}
  graph2::insertData .cb {1 4 3 2} {1 4 3 2} -color red -legend,title "Red"
  graph2::insertData .cb {1 2 3 4} {5 4 3 2} -color blue -dataset 2\
    -legend,title "Blue"
  #   graph2::drawGraph .cb

  canvas .cc -height 200 -width 300 -borderwidth 1 -relief ridge
  grid .cc -row 5 -column 2
  graph2::create .cc -style histogram -wid 10 -axis,x,nameProc {xLabel {{} une\
    deux trois quarte cinq}} -axis,y,nameProc {xLabel {{} unos dos tres quatro\
    cinco}} -rotate 1 -0,axis,x,LeftMargin 70 -axis,x,label "X Axis Label"\
    -axis,y,label "Y Axis Label" -legend,relx .9 -legend,rely .25\
    -legend,anchor ne

  graph2::insertData .cc {1 2 3 4} {1 2 3 4} -color red -legend,title "Red"
  graph2::insertData .cc {1 2 3 4} {3 3 2 2} -color blue -dataset 2\
    -legend,title "Blue"


  grid rowconfigure . 1 -weight 1
  grid columnconfigure . 1 -weight 1


  proc foo2 {cvs x y} {
    global shipData
    incr shipData(pos) 10
    set x [+ $x 4]
    set y [+ $y 30]

    array set colors {flood blue fire red list green pitch green}
    foreach ind [array names shipData *.$shipData(pos)] {
      foreach {id num} [split $ind .] {
	break
      }
      switch $id {
      fire {
	  $cvs create line [- $x 4] $y [- $x 4] [- $y $shipData($ind)]\
	    -fill $colors($id) -width 2
	}
      flood {
	  $cvs create line [- $x 1] $y [- $x 1] [- $y $shipData($ind)]\
	    -fill $colors($id) -width 2
	}
      list {
	  $cvs create line [- [+ $x 5] $shipData($ind)] $y [+ [+ $x 5]\
	    $shipData($ind)] [- $y 10] -fill $colors($id) -width 2
	}
      pitch {
	  $cvs create line [+ $x 2] [- [- $y 5] $shipData($ind)] [+ $x 11]\
	    [+ $shipData($ind) [- $y 5]] -fill $colors($id) -width 2
	}
      default {
	  puts "DUnno $id"
	}
      }
    }
  }
  array set shipData {
  flood.10 10 flood.20 15 flood.30 20 flood.40 25 flood.50 25 flood.60 20
  fire.10 5 fire.20 15 fire.30 20 fire.40 10 fire.50 5 fire.60 0
  list.10 0 list.20 1 list.30 2 list.40 3 list.50 2 list.60 1
  pitch.10 0 pitch.20 1 pitch.30 2 pitch.40 3 pitch.50 2 pitch.60 1
  pos 0
}
  canvas .ca -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .ca -row 5 -column 0
  graph2::create .ca -axis,y,dash {1 3} -axis,x,dash {3 2} -axis,y,min 0\
    -axis,x,min 0 -axis,x,maxTick 70 -legend,relx .1 -legend,rely .8\
    -legend,anchor sw -legend,font {times 16 bold}

  graph2::insertData .ca {10 20 30 40 50 60} { 100 90 70 75 80 85} -point foo2\
    -legend,title "compound data"


  canvas .cd -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .cd -row 6 -column 0
  graph2::create .cd -axis,y,use 0 -axis,x,use 0 -legend,relx .1\
    -legend,rely .8 -legend,anchor sw -style pie -colors {red green orange\
    blue yellow purple}

  graph2::insertData .cd {hundred ninety seventy sevfive eight0 eight5} { 100\
    90 70 75 80 85} -legend,title "Easy as Pi"

  canvas .ce -height 200 -width 600 -borderwidth 1 -relief ridge\
    -background white
  grid .ce -row 6 -column 1 -columnspan 2
  graph2::create .ce -axis,y,use 0 -axis,x,use 0 -style pie -legend,relx .9\
    -legend,rely .8 -legend,anchor se -0,axis,y,BottomMargin 40\
    -0,axis,y,TopMargin 30 -0,axis,x,LeftMargin 50 -0,axis,x,RightMargin 20


  graph2::insertData .ce {{User Error} {Enemy Action} {Windows BlueScreen}\
    {Funding Loss}} { 10 15 60 20} -legend,title "Reason for Ship Loss"

  canvas .cf -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .cf -row 0 -column 3

  graph2::create .cf -style histogram -wid 4 \
    -axis,y,nameProc {xLabel {{} unos dos tres quatro cinco six siente ocho}} \
    -axis,y,font {times 12 bold}\
    -rotate 1 -0,axis,x,LeftMargin 60 -histStyle single -legend,relx .75\
    -legend,rely .6 -legend,anchor nw
  graph2::insertData .cf {1 2 3 4} {1 2 3 4} -color red -legend,title "Red"
  graph2::insertData .cf {1 2 3 4} {3 3 2 2} -color blue -dataset 2\
    -legend,title "Blue"
  graph2::insertData .cf {1 2 3 4} {2 2 3 3} -color green -dataset 3\
    -legend,title "Green"

  canvas .cg -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .cg -row 1 -column 3

  graph2::create .cg -style histogram -wid 4 -axis,x,nameProc {xLabel {{} une\
    deux trois quarte cinq six sept}} -rotate 0 -0,axis,x,LeftMargin 60\
    -histStyle single -legend,relx .2 -legend,rely .05 -legend,anchor nw
  graph2::insertData .cg { 2 3 4 5} {1 2 3 4} -color red -legend,title "Red"
  graph2::insertData .cg {2 4 5} {2 3 4} -color green -dataset 2\
    -legend,title "Green"
  graph2::insertData .cg {1 2 3 4} {3 3 2 2} -color blue -dataset 3\
    -legend,title "Blue"

  set f1 [frame .fr1]
  grid $f1 -row 2 -column 3


  set cvs [canvas $f1.ch -height 60 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 1 -column 1

  graph2::create $cvs -style histogram -wid 8 -axis,x,nameProc {xLabel {{} une\
    deux trois quarte cinq six sept}} -axis,y,nameProc {xLabel {{} unos dos\
    tres quatro cinco}} -rotate 1 -0,axis,x,LeftMargin 60 -histStyle single\
    -legend,relx .99 -legend,rely .05 -legend,anchor ne
  graph2::insertData $cvs {1 2} {1 1} -color red -legend,title "Red"
  graph2::insertData $cvs {1 2} {3 3} -color blue -dataset 2\
    -legend,title "Blue"

  proc MakeYellow {cvs X Y xo yo id} {
    puts "  $cvs itemconfigure $id -fill Yellow"
    $cvs itemconfigure $id -fill Yellow
  }
  set cvs [canvas $f1.ci -height 60 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 2 -column 1
  graph2::create $cvs -style histogram -wid 4 -axis,y,nameProc {xLabel {{}\
    {}}} -axis,x,nameProc {xLabel {{} {} {} {} Red-Blu-Grn-Orng-NoMargin}}\
    -rotate 1 -histStyle single -axis,x,minTick 0 -legend,relx 1.99\
    -legend,rely .05 -legend,anchor ne -axis,x,label "RightClick Makes yellow"\
    -bind,bar,<ButtonPress-3> MakeYellow

  graph2::insertData $cvs {1 } {1 } -color red -legend,title "Red"
  graph2::insertData $cvs {1 } {3 } -color blue -dataset 2 -legend,title "Blue"
  graph2::insertData $cvs {1 } {3 } -color green -dataset 3\
    -legend,title "Green"
  graph2::insertData $cvs {1 } {3 } -color orange -dataset 4\
    -legend,title "Orange"

  set cvs [canvas $f1.cj -height 60 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 3 -column 1
  graph2::create $cvs -style histogram -wid 4 -axis,y,nameProc {xLabel\
    {{} 4-bars }} -axis,x,nameProc {xLabel\
    {{} Red-Blue-Green-Black-60LeftMargin }} -rotate 1 -0,axis,x,LeftMargin 60\
    -axis,x,minTick 0 -legend,relx 1.99 -legend,rely .05 -legend,anchor ne
  graph2::insertData $cvs {1 } {1 } -color red -legend,title "Red"
  graph2::insertData $cvs {1 } {3 } -color blue -dataset 2 -legend,title "Blue"
  graph2::insertData $cvs {1 } {3 } -color green -dataset 3\
    -legend,title "Green"
  graph2::insertData $cvs {1 } {3 } -color black -dataset 4\
    -legend,title "Black"


  set f2 [frame .f2]
  grid $f2 -row 5 -column 3

  set cvs [canvas $f2.ck -height 200 -width 100 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 1 -column 1
  graph2::create $cvs -style histogram -wid 4 -axis,x,nameProc {xLabel\
    {{} Red/Blue}} -rotate 0 -axis,x,minTick 0 -legend,relx 1.99\
    -legend,rely .05 -legend,anchor ne
  graph2::insertData $cvs {1 } {1 } -color red -legend,title "Red"
  graph2::insertData $cvs {1 } {3 } -color blue -dataset 2 -legend,title "Blue"


  set cvs [canvas $f2.cl -height 200 -width 150 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 1 -column 2
  graph2::create $cvs -style histogram -wid 12 -axis,x,nameProc {xLabel\
    {{} R/Blu/G/Blk }} -rotate 0 -0,axis,x,LeftMargin 80 -histStyle single\
    -axis,x,minTick 0 -0,axis,y,BottomMargin 40 -legend,relx 1.99 -legend,rely\
    -1.5 -legend,anchor ne
  graph2::insertData $cvs {1 } {1 } -color red -legend,title "Red"
  graph2::insertData $cvs {1 } {3 } -color blue -dataset 2 -legend,title "Blue"
  graph2::insertData $cvs {1 } {3 } -color green -dataset 3\
    -legend,title "Green"
  graph2::insertData $cvs {1 } {3 } -color black -dataset 4\
    -legend,title "Black"

  canvas .cm -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white
  grid .cm -row 0 -column 4
  graph2::create .cm -style area -wid 4 -rotate 0 -legend,relx 1.99\
    -legend,rely -1.5 -legend,anchor ne

  graph2::insertData .cm {1 2 3 4 5 6} {1 3 3 5 7 7} -color red -dataset 1\
    -legend,title "Area1"

  set cvs [canvas .cn -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 1 -column 4
  graph2::create $cvs -style area -wid 4 -rotate 0 -legend,relx 1.99\
    -legend,rely -1.5 -legend,anchor ne

  graph2::insertData $cvs {1 2 3 4 5 6} {1 3 3 5 7 7} -color red -dataset 1\
    -legend,title "Area1"

  graph2::insertData $cvs {1 2 3 4 5 6} {8 8 9 11 10 10} -color blue\
    -dataset 2 -legend,title "Area2"


  set cvs [canvas .co -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 2 -column 4
  graph2::create $cvs -style area -wid 4 -rotate 0 -legend,relx 1.99\
    -legend,rely -1.5 -legend,anchor ne

  graph2::insertData $cvs {1 2  4 5 6} {1 3  5 7 7} -color red -dataset 1\
    -legend,title "Area1"

  graph2::insertData $cvs {1  3 4 5 6} {8  9 11 10 10} -color blue -dataset 2\
    -legend,title "Area1"


  set cvs [canvas .cp -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 5 -column 4
  graph2::create $cvs -style area -axis,x,format "%.0f" -axis,y,format "%.0f"\
    -rotate 0 -legend,title {3 Sparse DataSets} -legend,x 10 -legend,y 10\
    -legend,anchor nw

  graph2::insertData $cvs {1 2 4 5 6} {1 3 5 7 7} -color red -dataset 1\
    -legend,title "Red Area"

  graph2::insertData $cvs {1 3 4 5 6} {8 9 11 10 10} -color blue -dataset 2\
    -legend,title "Blue Area"

  graph2::insertData $cvs {1 2 4 6} {2 2 2 2} -color green -dataset 3\
    -legend,title "Green Area"


  set cvs [canvas .cq -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 6 -column 4
  graph2::create $cvs -style area -rotate 0 -legend,title {3 Sparse DataSets}\
    -axis,x,format "%.0f" -axis,y,format "%.0f" -legend,x 10 -legend,y 10\
    -legend,anchor nw

  graph2::insertData $cvs {91 99 101 113 120} {1 3 5 7 7} -color red\
    -dataset 1 -legend,title "Red Area"

  graph2::insertData $cvs {47 48 49 51 120 } {8 9 11 10 10} -color blue\
    -dataset 2 -legend,title "Blue Area"

  graph2::insertData $cvs {18 19 38 120} {2 2 2 2} -color green -dataset 3\
    -legend,title "Green Area"

  set cvs [canvas .cr -height 200 -width 300 -borderwidth 1 -relief ridge\
    -background white]
  grid $cvs -row 6 -column 3
  graph2::create $cvs -style colorbar -wid 4 -axis,y,nameProc {xLabel\
    {{} Forwards Backwards DownUp}} -rotate 1 -legend,title {3 Rainbows}\
    -axis,x,LeftMargin 70 -legend,x 10 -legend,y 10 -legend,anchor nw\
    -colors {red orange yellow green blue violet black}\
    -bind,bar,<ButtonPress-3> ArgPrint

  proc ArgPrint {args} {
    puts "ARGS: $args"
  }

  graph2::insertData $cvs {1 1 1 1 1 1 1} {1 2 3 4 5 6 7} -dataset 1 -z {1 2 3\
    4 5 6 7} -legend,title "FirstSet"

  graph2::insertData $cvs {2 2 2 2 2 2 2} {2 3 4 5 6 7 8} -dataset 2 -z {7 6 5\
    4 3 2 1} -legend,title "Backwards"

  graph2::insertData $cvs {3 3 3 3 3 3 3 3 3 3 3 3 3 3} {1 1.5 2 2.5 3 3.5 4\
    4.5 5 5.5 6 6.5 7 7.5} -dataset 3 -z {7 6 5 4 3 2 1 1 2 3 4 5 6 7}\
    -legend,title "DblTime"


  grid rowconfigure . 1 -weight 1
  grid columnconfigure . 1 -weight 1
}

if {[info exists argv] &&($argv == "-testF")} {
  puts "BWid: [package require BWidget]"
  NoteBook .nb -height 460 -width 680
  grid .nb -row 1 -column 1
  source data.tcl

  proc cmp {a b} {
    return [expr {[lindex [split $a ,] end] > [lindex [split $b ,] end]}]

    # The previous line is 15% faster than the following lines.
    # foreach {file fld timea} [split $a ,] {break;}
    # foreach {file fld timeb} [split $b ,] {break;}
    # return [expr {$timea > $timeb}]
  }

  foreach i [lsort -command cmp [array names Data]] {
    foreach {file fld time} [split $i ,] {
      break
    }
    lappend vals($file,$fld,x) $time
    lappend vals($file,$fld,y) $Data($i)

    if {![info exists Graphs($file)]} {
      set Graphs($file) [.nb insert end $file -text $file]
      set fr [.nb getframe $file]
      puts "$file -- $fr"
      canvas $fr.c -height 400 -width 600 -bg white
      pack $fr.c
      update idletasks
    }
  }

  set colors {red blue orange green brown black red}
  set points {o + x box o + x box o + x box}

  foreach file [array names Graphs] {
    set fr [.nb getframe $file]
    set cvs $fr.c
    set ypos 40
    set col 1
    foreach pos [array names vals $file,*,x] {
      foreach {f c x} [split $pos ,] {
	break
      }
      graph2::create $cvs -0,axis,y,use 0
      graph2::insertData $cvs $vals($f,$c,x) $vals($f,$c,y) -dataset $col\
	-color [lindex $colors $col] -legend,title $c -point [lindex $points\
	$col] -axis,y,use 0
      #    graph2::drawGraph $cvs
      incr col
      incr ypos 20
    }
  }
}

