#####################################################################
# NAME:   help.tcl
# AUTHOR: Clif Flynt
# DATE:   10/2005
# DESC:   Help library
#         
# PARAMETERS:  
#  many and varied
#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2005
#
#   Contact: clif@noucorp.com
#
#####################################################################
set release(help.tcl) {$Header: /home/cvs/dataGraph/help.tcl,v 1.2 2005/10/02 13:23:38 clif Exp $}

namespace eval ::help {

# Pop up the "About" dialog box.
#
proc about {{parent .}} {
  variable helps
  catch {destroy .about}
  toplevel .about
  wm title .about About
  wm iconname .about About
  wm withdraw .about
  wm protocol .about WM_DELETE_WINDOW {.about.bb.can invoke}
  
  text .about.txt -yscrollcommand ".about.ysb set"
  scrollbar .about.ysb -command ".about.txt yview" -orient vertical
  HMinit_win .about.txt
  HMparse_html $helps(about) "HMrender .about.txt"

  grid .about.txt -row 0 -column 0
  grid .about.ysb -row 0 -column 1 -sticky ns
  button .about.bb -text Done -command {destroy .about}
  grid .about.bb -row 1 -column 0

  ::update idletasks
  centerWindow .about $parent
}

################################################################
# proc versions {}--
#    Display the version info for all modules with a "release"
#  global variable
# Arguments
#   None
# 
# Results
#   New toplevel window is created.
# 
proc versions {} {
  global release
  global tcl_platform
  catch {destroy .versions}
  toplevel .versions
  wm withdraw .versions

  grid [text .versions.t] -row 0 -column 0
  grid [button .versions.b -text OK -command {destroy .versions}]

  foreach n [array names release] {
    set l "$n: [lindex $release($n) 2]"
    .versions.t insert end "$l\n"
  }
  centerWindow .versions .
}

################################################################
# proc help {{helpType {}} {buttonBar {}} }--
#    Display help
# Arguments
#   helpType	Identifier into help screens
#   buttonBar   The frame that holds state buttons
#               If no helpType, this is used to identify the current
#               state for context-specific help
# 
# Results
#   New toplevel window is created.
# 
proc help {{helpType {}} {buttonBar {}} } {
  variable helps
  global tcl_platform
  set helps(_stack) ""

  catch {destroy .help}
  toplevel .help

  wm withdraw .help

  set w [text .help.t -yscrollcommand {.help.sb set}]
  grid $w -row 1 -column 1 -sticky news
  grid columnconfigure .help 1 -weight 1
  grid rowconfigure .help 1 -weight 1
  HMinit_win $w
  
  set w [scrollbar .help.sb -orient vertical -command {.help.t yview}]
  grid $w -row 1 -column 2 -sticky ns
  
  set f [frame .help.f1]
  grid $f -row 0 -column 1 -columnspan 2 -sticky ew
  
  set w [button $f.back -image icon:back -command {help::_back} -relief flat]
  pack $w -side left
  
  if {![string equal $tcl_platform(platform) windows]} {
    set w [button $f.q -image icon:del -command {destroy .help} -relief flat]
    pack $w -side right
  }

  set page generic
  if {[string match $helpType ""]} {
    if {![string equal {} $buttonBar]} {
      foreach w [winfo children $buttonBar] {
        if {[string match [$w cget -relief] sunken]} {
          set page $w
	}
      }
    }
      _insertHelp $page
  } else {
      _insertHelp $helpType
  }
  centerWindow .help .
}

################################################################
# proc _back {}--
#    Return to previous page
# Arguments
#   None
# 
# Results
#   Changes stack and moves back to previous help page
# 
proc _back {} {
  variable helps
  if {[llength $helps(_stack)] == 1} {return}
  # Clear the current page
  set helps(_stack) [lrange $helps(_stack) 0 end-1]
  set bck [lindex $helps(_stack) end]
  set helps(_stack) [lrange $helps(_stack) 0 end-1]
  _insertHelp $bck
}

################################################################
# proc _insertHelp {id}--
#    Put the help into the window
# Arguments
#   id 		Identifier for this help text
# 
# Results
#   Window is modified.
# 
proc _insertHelp {id} {
  variable helps
  
  .help.t configure -state normal

  if {[string equal $helps(_stack) ""]} {
    .help.f1.back configure -state disabled
  } else {
    .help.f1.back configure -state normal
  }

  lappend helps(_stack) $id

  .help.t delete 0.0 end

  if {[info exists helps($id)]} {
    # .help.t insert end $helps($id)
    set n [lindex [split [string trim $id .] .] end]
    set n [string map {_ { }} $n]
    wm title .help $n

    HMparse_html $helps($id) "HMrender .help.t"
  } else {
    .help.t insert end $helps(generic)
  }
    .help.t configure -state disabled
}

# End of the ::help namespace

set helps(about) {
The <B>dataGraph</B> extracts data from ascii (or gzipped) files
and generates line, scatter, histogram, pie or area graphs from
the appropriate parts.
<P>
It was developed by Clif Flynt of Noumena Corporation 
(clif@cflynt.com, http://www.noucorp.com) and is released into the
wilds under the Berkely License.
<P>
Copyright September, 2005
}


set helps(INDEX) {
  HTML TEXT HERE
}

set helps(icon:print) {
  Prints the graph window.
}

set helps(generic) {
  <B>dataGraph</B> will extract numeric fields from flat ASCII
files and CSV data.
<P>
  It can select only lines that match a particular
pattern to include in the data.  
<P>
It can graph XY data as line, scatter, histogram or area graphs.
<P>
It can graph Y data as pie chargs.
<P>
It can graph XYZ data as color bar charts.
<P>
}

set helps(concepts,Graph_Types) {
The <B>dataGraph</B> application can generate several types of graphs
depending on the data provided.  
<B>dataGraph</B> can graph XY data as 
<A HREF="concepts,line">line</A>, 
<A HREF="concepts,scatter">scatter</A>, 
or <A HREF="concepts,area">area</A> graphs.
<P>
<B>dataGraph</B> can graph Y data as 
<A HREF="concepts,pie">pie</A> charts, using the X field as
slice labels.
<P>
<B>dataGraph</B> can graph XYZ data as 
<A HREF="concepts,colorbar">colorbar</A> charts.
}


set helps(concepts,dataset) {
  Datasets are selected from the Data Definitions pages.
<P>
Select the data set you wish to define (simplest to start
with 1, then 2, etc for multiple lines).
<P>
Each dataset:
 <UL>
  <LI> Can have a title assigned in the legend.
  <LI> can be a different width.
  <LI> can be scaled to the X or Y axis (or to fill the graph).
  <LI> can have an X, Y and/or Z component.
 </UL>
 
 The <A HREF="data_Definition,Legend_Title"> Legend Title</A> field
holds the string that will be displayed on the graphs legend 
window.  For Pie charts, this string is also displayed under the pie.
<P>
The <A HREF="data_Definition,Width">Width</A> field holds a number that
will be the line width for line graphs, or the bar width for
histograms.  The units are pixels.
<P>
The <A HREF="data_Definition,Use_Axis"> Use X Axis and Use Y Axis</A>
checkbuttons select whether the data should be scaled to fit on
the axis with other data (when selected) or should be scaled to
fill the graph in that dimension.  Data may also be scaled with
the <A HREF="data_Definition,Scale_By">Scale By</A> entry.
<P>
The <A HREF="data_Definition,Data_Source">Data Source</A> entry is where
you enter the name of a text file that has the data to plot.
The Field and Separator entries allow you do define which field
to display along this axis, and how fields are distinguished
in the file.
<P>
The <A HREF="data_Definition,Line_Match">Line Match Pattern</A> will
select only lines that match that pattern to be included in the
graph.
}
set helps(data_Definition,Data_Source) {
  The <B>Data Source</B> entry lets you define a file that contains the
ASCII data to be graphed.  
<P>
A normal file can be selected with the <B>Browse</B> button.
<P>
You can graph program output by entering a shell command preceeded
by a | (vertical bar) symbol.

For example, a Data Source of <BR>
<CODE>| grep identifier /tmp/datafile</CODE>
<BR>
Is the equivalent of selecting <CODE>/tmp/datafile</CODE> in the
<B>Data Source</B> entry and <CODE>identifier</CODE> in the 
<B>Line Match Pattern</B>.
  
  
}
set helps(data_Definition,Legend_Title) {
Each data set has a separate title that is displayed in
the legend window.
<P>
 The Legend Title field
holds the string that will be displayed on the graphs legend 
window.  For Pie charts, this string is also displayed under the pie.
<P>
This may be any number of characters, though longer ones will 
take up more space and may be unreadable.
}

set helps(data_Definition,Width) {
Each data set can be drawn with a separate width.
<P>
The Width field holds a number that
will be the line width for line graphs, or the bar width for
histograms.  The units are pixels.
<P>
}
set helps(data_Definition,Use_Axis) {
Each data set can be drawn so that it is scaled ot a common X or
Y axis (useful when you need to compare the values of N datasets), or
it can be scaled to fill the graph in that axis (useful when you are
comparing the shape of N datasets.)
<P>
The Use X Axis and Use Y Axis
checkbuttons select whether the data should be scaled to fit on
the axis with other data (when selected) or should be scaled to
fill the graph in that dimension.  Data may also be scaled with
the <A HREF="data_Definition,Scale_By">Scale By</A> entry.
}

set helps(data_Definition,Scale_By) {
Numeric data will be modified by the arithmetic value in this field.  
By default, the value is <CODE> * 1.0</CODE>, no scaling takes place.  
Any valid Tcl numeric string, real, or exponential may be placed in this field. 
<P>
The numeric string will be preceeded by the value read after formatting
when being calculated.
<P>
For instance, a value of <CODE> * 1.0E3</CODE>
 would multiply the values by 1000.
}
set helps(data_Definition,Separator) {
  The Separator entry defines the character or characters used
to separate fields in the data file.
<P>
  The default value is a comma, to select fields from comma
separated files, as generated by Open Office, and as exported by
Excell.  Any ASCII value can be entered here.
<P>
  If you enter multiple
values, each will be used as a separator, not the combination of
these characters.
}
set helps(data_Definition,Field) {
  The Field entry defines the field to be selected as data from
each line.  The fields are numbered from 0.
<P>
  For instance, if the input data were:
<PRE>
a,2,3
b,3,4
c,4,5
</PRE>
  A Separator of "," and a Field of 1 would select 2, 3, and 4
as data values.
}

set helps(data_Definition,Process) {
  Data that is read into dataGraph can be <I>processed</I>
before being graphed.  The options are
  <DL>
    <DT> actual
    <DD> Use the values, just like you'd expect
    <DT> count
    <DD> Graph the count of lines that match the selection qualities.
      This generates a histogram "Y" data. for example
    <DT> index
    <DD> Record the indices of data that have been counted.  This generates
     an "X" histogram data.
  </DL>
}
set helps(data_Definition,Format) {
  Data can be in one of several formats.  
  <DL>
    <DT> Empty
    <DD> Treat as numeric data
    <DT> time
    <DD> Scan as a time and convert to seconds.  All times happen today.
    <DT> date
    <DD> Scan as a full date and convert to seconds since the epoch.  All times are
      based on the local system date of "the epoch".
    <DT> <I>blank</I> (The default)
    <DD> Treat as numeric data
  </DL>
}
set helps(data_Definition,Line_Match) {
The Line Match Pattern will
select only lines that match that pattern to be included in the
graph.  Any line that includes the pattern will be used, while lines
that do not include the pattern will be skipped.
<P>
This is useful when you have multiple sets of data intermingled
and you only want to select specific lines.
<P>
For example given data:
<PRE>
bill,2,3
bob,3,4
bill,4,5
</PRE>
<P>
You could select only lines concerning "bill" to graph by
putting "bill" in the Line Match Pattern field.
<P>
The pattern will be evaulated using Tcl glob pattern rules.
}
set helps(concepts,line) {
  Line graphs are generated by <B>line</B> in the 
<B>Graph Description/Style</B> page and selecting at least
one set of X and Y values in a <B>Data Definition</B> dataset.
}

set helps(concepts,line) {
Line graphs are generated by selecting <B>line</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of X and Y values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
}

set helps(concepts,histogram) {
Histogram graphs are generated by selecting <B>histogram</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of X and Y values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
}

set helps(concepts,scatter) {
Scatter graphs are generated by selecting <B>scatter</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of X and Y values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
}

set helps(concepts,area) {
Area graphs are generated by selecting <B>area</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of X and Y values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
}

set helps(concepts,pie) {
Pie charts are generated by selecting <B>pie</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of Y values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
<P>
If an X data source is selected, these values will be displayed
in the legend.
<P>
For example, given data:
<PRE>
peter,2,3
paul,3,4
mary,4,5
</PRE>
Selecting field 0 for X, and field 1 for Y would create a pie chart
with the last pie slice twice as large as the first, and the middle
in between.  The legend would define the smallest pie slice as peter,
and the largest as mary.

}

set helps(concepts,colorbar) {
Colorbar graphs are generated by selecting <B>colorbar</B> in the  <B>Graph
Description/Style</B> page and selecting at least one set of X, Y and Z values
in a <A HREF="concepts,dataset">Data Definition</A> dataset.
<P>
The X values will define the location of the bar.  
<P>
The Y values will define the size of each colored section of a bar.
<P>
The Z values will define the color of each section of the bar.
For example, given data:
<PRE>
1,2,1
1,3,2
2,3,2
2,4,1
3,1,1
3,1,2
</PRE>
Selecting field 0 as X, field 1 as Y and field 3 as Z, would create
3 bars (at X location 1, 2, and 3) each of which would have 2 colored
sections (like single style histograms).  The second colored section
would be larger in the bar at X=1, smaller in the bar at X=2, and
the same size in the bar at X=3.

}

set helps(graph_Description,Generic) {
  The Generic page supports options that are generic to
all graphs.  
<DL>
  <DT> Graph Style
  <DD> The <B>dataGraph</B> application can generate several types of graphs
      depending on the field selected in the Graph Description/Style
      page and the data provided.
      <B>dataGraph</B> can graph XY data as 
      <A HREF="concepts,line">line</A>, 
      <A HREF="concepts,scatter">scatter</A>, 
      or <A HREF="concepts,area">area</A> graphs.
      <P>
      <B>dataGraph</B> can graph Y data as 
      <A HREF="concepts,pie">pie</A> charts, using the X field as
      slice labels.
      <P>
      <B>dataGraph</B> can graph XYZ data as 
      <A HREF="concepts,colorbar">colorbar</A> charts.
  <DT> Rotation
  <DD> Will rotate the graph 90 degrees.  Histogram and colorbar graphs
   become horizontal instead of vertical, and X/Y axes are swapped
   for line and scatter graphs.
  <DT> X axis label
  <DD> Text to draw below the X axis
  <DT> Y axis label
  <DD> Text to draw to the left of the Y axis.  Words will be split
  onto separate lines.
</DL>
}

set helps(graph_Description,Line/Scatter) {
  The Line/Scatter Settings page supports options that are specific to
line and scatter graphs.  
<DL>
  <DT> drawPoint
  <DD> Selecting this will draw markers where data exists.  Unselecting
     this will only draw the lines connecting the points.
  <DT> dash
  <DD> The dash pattern to use for drawing the lines.  An empty string is
  a solid line, combinations of _, -, and . to define long, short and dot
  sized dashes.
  <DT> width
  <DD> The width of the lines to draw.
  <DT> color
  <DD> A color to use for this line.  You may select from the pulldown
  menu, or type in a legitamate Tk color value.
  <DT> point
  <DD> The point to draw for scatter graphs, or at data locations when
    <B>draPoint</B> is set.  Select from pulldown only.
</DL>
}

set helps(graph_Description,Histogram) {
  The Histogram Settings page supports options that are specific to
Histogram and Colorbar graphs.  
<DL>
  <DT> Show Values
  <DD> When histShowAbsolute is set the values of the bars are printed
  at the top of the bar.

  <DT> Histogram Style
  <DD> When plotting multiple datasets, the data for a single base
    location may be <B>stacked</B> (to put bars side by side) or 
       <B>single</B> to draw a single bar with one value on top
       (or next to, if rotate) the other.

  <DT> width
  <DD> The width of the bar

  <DT> color
  <DD> The color for bars this dataset.  Only used for histograms.  The
    colorbars are hard-coded for red-violet rainbow.
</DL>
}

set helps(graph_Description,Axis) {
  The Axis Settings page supports options that are specific to
setting the axis that is drawn.
<DL>
  <DT> Axis Color
  <DD> Selects the color to draw the axis.  Defaults to black.  You
  may type in any valid Tk color.

  <DT> Label Color
  <DD> Selects the color to draw the axis label.  Defaults to black.  You
  may type in any valid Tk color.
  
  <DT> Format
  <DD> The labels on the axes can be displayed either as <CODE>numeric</CODE> values
  (the default) or converted to <CODE>date</CODE> format.  The 
  formatting is done using the <CODE>format</CODE> command for numeric
  formatting, and <CODE>clock format</CODE> for dates.

  <DT> format String
  <DD> The format string controls how the axis labels will be formatted.
    For <CODE>numeric</CODE> formatted data, use strings similar to those
    used by printf.  For <CODE>date</CODE> formatted strings, use strings
    like those used in the <CODE>date</CODE> command.  See 
    <CODE>man n format</CODE> and <CODE>man n clock</CODE> or the
    Windows Tcl/Tk Help for details.
    
  <DT> Scaler String
  <DD> Data that has been scaled when it is read may need to be scaled
    back when used as an axis label.  This string should be the inverse
    of whatever Scaling was done in the <B>Data Definition</B> window.
    <P>
    This strig is appended to the  internal value of the data before
    arithmetic processing.  
    <P>
    For example, if you are reading time/date values, and converting
    seconds to days, you'd have <CODE>/ 86400</CODE> in the Data Definition
    <B>Scale By</B> entry.
    <P>
    To get the correct days listed on the axes, you'd put <CODE>* 86400</CODE>
    in the Axis/Scaler String window.

  <DT> Left Margin
  <DD> The amount of space in pixels to leave to the left of the graph before
    starting the data.

  <DT> Right Margin
  <DD> The amount of space in pixels to leave to the right of the graph before
    starting the data.

  <DT> Top Margin
  <DD> The amount of space in pixels to leave to the top of the graph before
    starting the data.

  <DT> Bottom Margin
  <DD> The amount of space in pixels to leave to the bottom of the graph before
    starting the data.
</DL>
}
}
